@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2013 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -28,6 +28,7 @@ import signal
 import socket
 import platform
 import time
+import re
 
 try:
     import Cheetah
@@ -259,7 +260,7 @@ def print_version():
     print """
 %s-%s
 
-Copyright (C) 2008-2012, The SABnzbd-Team <team@sabnzbd.org>
+Copyright (C) 2008-2013, The SABnzbd-Team <team@sabnzbd.org>
 SABnzbd comes with ABSOLUTELY NO WARRANTY.
 This is free software, and you are welcome to redistribute it
 under certain conditions. It is licensed under the
@@ -671,7 +672,7 @@ def get_webhost(cherryhost, cherryport, https_port):
     return cherryhost, cherryport, browserhost, https_port
 
 
-def attach_server(host, port, cert=None, key=None):
+def attach_server(host, port, cert=None, key=None, chain=None):
     """ Define and attach server, optionally HTTPS
     """
     if not (sabnzbd.cfg.no_ipv6() and '::1' in host):
@@ -680,20 +681,18 @@ def attach_server(host, port, cert=None, key=None):
         if cert and key:
             http_server.ssl_certificate = cert
             http_server.ssl_private_key = key
+            http_server.ssl_certificate_chain = chain
         adapter = _cpserver.ServerAdapter(cherrypy.engine, http_server, http_server.bind_addr)
         adapter.subscribe()
 
 
-def is_sabnzbd_running(url):
+def is_sabnzbd_running(url, timeout=None):
     """ Return True when there's already a SABnzbd instance running.
     """
     try:
         url = '%s&mode=version' % (url)
-        ver = sabnzbd.newsunpack.get_from_url(url)
-        if ver and ver.strip(' \n\r\t') == sabnzbd.__version__:
-            return True
-        else:
-            return False
+        ver = sabnzbd.newsunpack.get_from_url(url, timeout=timeout)
+        return bool(ver and re.search(r'\d+\.\d+\.', ver))
     except:
         return False
 
@@ -713,7 +712,7 @@ def find_free_port(host, currentport):
 
 
 def check_for_sabnzbd(url, upload_nzbs, allow_browser=True):
-    """ Check for a running instance of sabnzbd(same version) on this port
+    """ Check for a running instance of sabnzbd on this port
         allow_browser==True|None will launch the browser, False will not.
     """
     if allow_browser is None:
@@ -1350,6 +1349,10 @@ def main():
 
     https_cert = sabnzbd.cfg.https_cert.get_path()
     https_key = sabnzbd.cfg.https_key.get_path()
+    https_chain = sabnzbd.cfg.https_chain.get_path()
+    if not (sabnzbd.cfg.https_chain() and os.path.exists(https_chain)):
+        https_chain = None
+
     if enable_https:
         # If either the HTTPS certificate or key do not exist, make some self-signed ones.
         if not (https_cert and os.path.exists(https_cert)) or not (https_key and os.path.exists(https_key)):
@@ -1379,14 +1382,15 @@ def main():
                 # Extra HTTP port for secondary localhost
                 attach_server(hosts[1], cherryport)
                 # Extra HTTPS port for secondary localhost
-                attach_server(hosts[1], https_port, https_cert, https_key)
+                attach_server(hosts[1], https_port, https_cert, https_key, https_chain)
             cherryport = https_port
         elif multilocal:
             # Extra HTTPS port for secondary localhost
             attach_server(hosts[1], cherryport, https_cert, https_key)
 
         cherrypy.config.update({'server.ssl_certificate' : https_cert,
-                                'server.ssl_private_key' : https_key })
+                                'server.ssl_private_key' : https_key,
+                                'server.ssl_certificate_chain' : https_chain})
     elif multilocal:
         # Extra HTTP port for secondary localhost
         attach_server(hosts[1], cherryport)
@@ -1404,6 +1408,17 @@ def main():
     else:
         sessions = None
 
+    mime_gzip = ('text/html',
+                 'text/plain',
+                 'text/css',
+                 'text/xml',
+                 'text/javascript',
+                 'application/javascript',
+                 'text/x-javascript',
+                 'application/x-javascript',
+                 'text/x-json',
+                 'application/json'
+                 )
     cherrypy.config.update({'server.environment': 'production',
                             'server.socket_host': cherryhost,
                             'server.socket_port': cherryport,
@@ -1413,7 +1428,7 @@ def main():
                             'engine.reexec_retry' : 100,
                             'tools.encode.on' : True,
                             'tools.gzip.on' : True,
-                            'tools.gzip.mime_types' : ['text/html', 'text/plain', 'text/javascript', 'text/css', 'application/x-javascript'],
+                            'tools.gzip.mime_types' : mime_gzip,
                             'tools.sessions.on' : bool(sessions),
                             'tools.sessions.storage_type' : 'file',
                             'tools.sessions.storage_path' : sessions,
@@ -1504,8 +1519,8 @@ def main():
         if sabnzbd.FOUNDATION:
             import sabnzbd.osxmenu
             sabnzbd.osxmenu.notify("SAB_Launched", None)
-        growler.send_notification('SABnzbd %s' % (sabnzbd.__version__),
-                             "http://%s:%s/sabnzbd" % (browserhost, cherryport), 'startup')
+        growler.send_notification('SABnzbd%s' % growler.hostname(),
+                                  T('SABnzbd %s started') % sabnzbd.__version__, 'startup')
         # Now's the time to check for a new version
         check_latest_version()
     autorestarted = False
@@ -1548,7 +1563,7 @@ def main():
             add_local(f)
 
     # Have to keep this running, otherwise logging will terminate
-    timer = 0
+    timer = timer5 = 0
     while not sabnzbd.SABSTOP:
         if sabnzbd.WIN_SERVICE:
             rc = win32event.WaitForMultipleObjects((sabnzbd.WIN_SERVICE.hWaitStop,
@@ -1574,7 +1589,7 @@ def main():
         ### 30 sec polling tasks
         if timer > 9:
             timer = 0
-            # Keep Windows awake (if needed)
+            # Keep OS awake (if needed)
             sabnzbd.keep_awake()
             # Restart scheduler (if needed)
             scheduler.restart()
@@ -1588,6 +1603,15 @@ def main():
             if sabnzbd.WIN_SERVICE and mail:
                 mail.send('active')
 
+            if timer5 > 9:
+                ### 5 minute polling tasks
+                timer5 = 0
+                if sabnzbd.cfg.web_watchdog() and not is_sabnzbd_running('%s/api?tickleme=1' % sabnzbd.BROWSER_URL, 120):
+                    autorestarted = True
+                    cherrypy.engine.execv = True
+            else:
+                timer5 += 1
+
         else:
             timer += 1
 
@@ -1813,4 +1837,4 @@ if __name__ == '__main__':
             main()
 
     else:
-        main()
\ No newline at end of file
+        main()
