@@ -31,6 +31,10 @@ import subprocess
 import time
 import cherrypy
 from threading import RLock, Lock, Condition, Thread
+try:
+    import sleepless
+except ImportError:
+    sleepless = None
 
 #------------------------------------------------------------------------
 # Determine platform flags
@@ -216,6 +220,13 @@ def initialize(pause_downloader = False, clean_up = False, evalSched=False, repa
         # New admin folder
         misc.remove_all(cfg.admin_dir.get_path(), '*.sab')
 
+    ### Optionally wait for "incomplete" to become online
+    if cfg.wait_for_dfolder():
+        wait_for_download_folder()
+    else:
+        cfg.download_dir.set(cfg.download_dir(), create=True)
+    cfg.download_dir.set_create(True)
+
     ### Set access rights for "incomplete" base folder
     misc.set_permissions(cfg.download_dir.get_path(), recursive=False)
 
@@ -298,10 +309,11 @@ def initialize(pause_downloader = False, clean_up = False, evalSched=False, repa
     PostProcessor()
 
     NzbQueue()
-    NzbQueue.do.read_queue(repair)
 
     Assembler()
 
+    NzbQueue.do.read_queue(repair)
+
     Downloader(pause_downloader or paused)
 
     DirScanner()
@@ -345,11 +357,6 @@ def start():
         logging.debug('Starting urlgrabber')
         URLGrabber.do.start()
 
-        if DARWIN_ML:
-            logging.debug('Starting OSX keep-awake service')
-            osxmenu.OsxAwake()
-            osxmenu.OsxAwake.do.start()
-
 
 @synchronized(INIT_LOCK)
 def halt():
@@ -409,9 +416,6 @@ def halt():
         except:
             logging.error('Fatal error at saving state', exc_info=True)
 
-        if DARWIN_ML:
-            osxmenu.OsxAwake.do.stop()
-
 
         # The Scheduler cannot be stopped when the stop was scheduled.
         # Since all warm-restarts have been removed, it's not longer
@@ -620,7 +624,7 @@ def add_nzbfile(nzbfile, pp=None, script=None, cat=None, priority=NORMAL_PRIORIT
             logging.info("Traceback: ", exc_info = True)
 
     if ext.lower() in ('.zip', '.rar'):
-        return ProcessArchiveFile(filename, path, pp, script, cat, priority=priority)
+        return ProcessArchiveFile(filename, path, pp, script, cat, priority=priority, nzbname=nzbname)
     else:
         return ProcessSingleFile(filename, path, pp, script, cat, priority=priority, nzbname=nzbname, keep=keep, reuse=reuse)
 
@@ -764,16 +768,21 @@ def empty_queues():
 
 
 def keep_awake():
-    """ If we still have work to do, keep Windows system awake
+    """ If we still have work to do, keep Windows/OSX system awake
     """
-    if KERNEL32 or DARWIN_ML:
-        if not sabnzbd.downloader.Downloader.do.paused:
-            if (not PostProcessor.do.empty()) or not NzbQueue.do.is_empty():
-                if KERNEL32:
-                    # set ES_SYSTEM_REQUIRED
-                    KERNEL32.SetThreadExecutionState(ctypes.c_int(0x00000001))
-                else:
-                    osxmenu.OsxAwake.do.stay_awake = True
+    if KERNEL32 or sleepless:
+        if sabnzbd.cfg.keep_awake():
+            awake = False
+            if not sabnzbd.downloader.Downloader.do.paused:
+                if (not PostProcessor.do.empty()) or not NzbQueue.do.is_empty():
+                    awake = True
+                    if KERNEL32:
+                        # set ES_SYSTEM_REQUIRED
+                        KERNEL32.SetThreadExecutionState(ctypes.c_int(0x00000001))
+                    else:
+                        sleepless.keep_awake(u'SABnzbd is busy downloading and/or post-processing')
+            if not awake and sleepless:
+                sleepless.allow_sleep()
 
 
 def CheckFreeSpace():
@@ -1036,6 +1045,9 @@ def check_all_tasks():
     # Check one-shot pause
     sabnzbd.scheduler.pause_check()
 
+    # Check (and terminate) idle jobs
+    sabnzbd.nzbqueue.NzbQueue.do.stop_idle_jobs()
+
     return True
 
 
@@ -1070,21 +1082,21 @@ def check_incomplete_vs_complete():
             cfg.download_dir.set('incomplete')
 
 
+def wait_for_download_folder():
+    """ Wait for download folder to become available """
+    while not cfg.download_dir.test_path():
+        logging.debug('Waiting for "incomplete" folder')
+        time.sleep(2.0)
+    
+
 # Required wrapper because nzbstuff.py cannot import downloader.py
 def active_primaries():
     return sabnzbd.downloader.Downloader.do.active_primaries()
 
 
-def proxy_postproc(nzo):
-    sabnzbd.postproc.PostProcessor.do.process(nzo)
-
 def proxy_pre_queue(name, pp, cat, script, priority, size, groups):
     return sabnzbd.newsunpack.pre_queue(name, pp, cat, script, priority, size, groups)
 
-def proxy_get_history_size():
-    history_db = sabnzbd.database.get_history_handle()
-    return history_db.get_history_size()
-
 def proxy_build_history():
     """ Proxy to let nzbqueue call api """
     return sabnzbd.api.build_history()
