@@ -52,7 +52,7 @@ from sabnzbd.utils.json import JsonWriter
 from sabnzbd.utils.pathbrowser import folders_at_path
 from sabnzbd.misc import loadavg, to_units, diskfree, disktotal, get_ext, \
                          get_filename, int_conv, globber, time_format, remove_all, \
-                         starts_with_path
+                         starts_with_path, cat_convert
 from sabnzbd.encoding import xml_name, unicoder, special_fixer, platform_encode, html_escape
 from sabnzbd.postproc import PostProcessor
 from sabnzbd.articlecache import ArticleCache
@@ -177,10 +177,11 @@ def _api_queue_delete_nzf(output, value, kwargs):
 
 
 def _api_queue_rename(output, value, kwargs):
-    """ API: accepts output, value(=old name), value2(=new name) """
+    """ API: accepts output, value(=old name), value2(=new name), value3(=password) """
     value2 = kwargs.get('value2')
+    value3 = kwargs.get('value3')
     if value and value2:
-        NzbQueue.do.change_name(value, special_fixer(value2))
+        NzbQueue.do.change_name(value, special_fixer(value2), special_fixer(value3))
         return report(output)
     else:
         return report(output, _MSG_NO_VALUE2)
@@ -293,9 +294,11 @@ def _api_addfile(name, output, kwargs):
         #Side effect of next line is that attribute .value is created
         #which is needed to make add_nzbfile() work
         size = name.length
-    else:
+    elif hasattr(name, 'value'):
         size = len(name.value)
-    if name is not None and name.filename and size:
+    else:
+        size = 0
+    if name is not None and size and name.filename:
         cat = kwargs.get('cat')
         xcat = kwargs.get('xcat')
         if not cat and xcat:
@@ -344,7 +347,7 @@ def _api_addlocalfile(name, output, kwargs):
 
                 if get_ext(name) in ('.zip', '.rar'):
                     res = sabnzbd.dirscanner.ProcessArchiveFile(\
-                        fn, name, pp=pp, script=script, cat=cat, priority=priority, keep=True)
+                        fn, name, pp=pp, script=script, cat=cat, priority=priority, keep=True, nzbname=nzbname)
                 elif get_ext(name) in ('.nzb', '.gz'):
                     res = sabnzbd.dirscanner.ProcessSingleFile(\
                         fn, name, pp=pp, script=script, cat=cat, priority=priority, keep=True, nzbname=nzbname)
@@ -422,6 +425,8 @@ def _api_history(name, output, kwargs):
     limit = kwargs.get('limit')
     search = kwargs.get('search')
     failed_only = kwargs.get('failed_only')
+    if not limit:
+        limit = cfg.history_limit()
 
     if name == 'delete':
         special = value.lower()
@@ -430,10 +435,10 @@ def _api_history(name, output, kwargs):
             history_db = cherrypy.thread_data.history_db
             if special in ('all', 'failed'):
                 if del_files:
-                    del_job_files(history_db.get_failed_paths())
-                history_db.remove_failed()
+                    del_job_files(history_db.get_failed_paths(search))
+                history_db.remove_failed(search)
             if special in ('all', 'completed'):
-                history_db.remove_completed()
+                history_db.remove_completed(search)
             return report(output)
         elif value:
             jobs = value.split(',')
@@ -554,10 +559,13 @@ def _api_auth(name, output, kwargs):
     if not cfg.disable_key():
         auth = 'badkey'
         key = kwargs.get('key', '')
-        if key == cfg.nzb_key():
-            auth = 'nzbkey'
-        if key == cfg.api_key():
+        if not key:
             auth = 'apikey'
+        else:
+            if key == cfg.nzb_key():
+                auth = 'nzbkey'
+            if key == cfg.api_key():
+                auth = 'apikey'
     elif cfg.username() and cfg.password():
         auth = 'login'
     return report(output, keyword='auth', data=auth)
@@ -625,6 +633,12 @@ def _api_watched_now(name, output, kwargs):
     return report(output)
 
 
+def _api_resume_pp(name, output, kwargs):
+    """ API: accepts output """
+    PostProcessor.do.paused = False
+    return report(output)
+
+
 def _api_rss_now(name, output, kwargs):
     """ API: accepts output """
     # Run RSS scan async, because it can take a long time
@@ -643,7 +657,7 @@ def _api_test_email(name, output, kwargs):
     pack['unpack'] = ['action 1', 'action 2']
     res = sabnzbd.emailer.endjob('I had a d\xe8ja vu', 123, 'unknown', True,
                                  os.path.normpath(os.path.join(cfg.complete_dir.get_path(), '/unknown/I had a d\xe8ja vu')),
-                                 123*MEBI, pack, 'my_script', 'Line 1\nLine 2\nLine 3\nd\xe8ja vu\n', 0)
+                                 123*MEBI, None, pack, 'my_script', 'Line 1\nLine 2\nLine 3\nd\xe8ja vu\n', 0)
     if res == 'Email succeeded':
         res = None
     return report(output, error=res)
@@ -788,6 +802,7 @@ _api_table = {
     'rescan'          : _api_rescan,
     'eval_sort'       : _api_eval_sort,
     'watched_now'     : _api_watched_now,
+    'resume_pp'       : _api_resume_pp,
     'rss_now'         : _api_rss_now,
     'browse'          : _api_browse,
     'reset_quota'     : _api_reset_quota,
@@ -1100,7 +1115,7 @@ def build_queue(web_dir=None, root=None, verbose=False, prim=True, webdir='', ve
             slot['mbdone_fmt'] = locale.format('%d', int(mb-mbleft), True)
         slot['size'] = format_bytes(bytes)
         slot['sizeleft'] = format_bytes(bytesleft)
-        if not Downloader.do.paused and status != 'Paused' and status != 'Fetching' and not found_active:
+        if not Downloader.do.paused and status not in (Status.PAUSED, Status.FETCHING) and not found_active:
             if status == Status.CHECKING:
                 slot['status'] = Status.CHECKING
             else:
@@ -1124,7 +1139,7 @@ def build_queue(web_dir=None, root=None, verbose=False, prim=True, webdir='', ve
             slot['percentage'] = "%s" % (int(((mb-mbleft) / mb) * 100))
         slot['missing'] = missing
 
-        if status in (Status.PAUSED, Status.CHECKING):
+        if Downloader.do.paused or Downloader.do.postproc or status not in (Status.DOWNLOADING, Status.QUEUED):
             slot['timeleft'] = '0:00:00'
             slot['eta'] = 'unknown'
         else:
@@ -1272,6 +1287,7 @@ def qstatus_data():
 
     status = {
         "state" : state,
+        "pp_active" : not PostProcessor.do.empty(),
         "paused" : Downloader.do.paused,
         "pause_int" : scheduler.pause_int(),
         "kbpersec" : bpsnow / KIBI,
@@ -1398,6 +1414,7 @@ def rss_qstatus():
         bytes = pnfo[PNFO_BYTES_FIELD] / MEBI
         mbleft = (bytesleft / MEBI)
         mb = (bytes / MEBI)
+        nzo_id = pnfo[PNFO_NZO_ID_FIELD]
 
 
         if mb == mbleft:
@@ -1415,6 +1432,8 @@ def rss_qstatus():
         else:
             item.link    = "http://%s:%s/sabnzbd/history" % ( \
             cfg.cherryhost(), cfg.cherryport() )
+            item.guid = nzo_id
+
         status_line  = []
         status_line.append('<tr>')
         #Total MB/MB left
@@ -1536,7 +1555,8 @@ def build_header(prim, webdir=''):
     if not color:
         color = ''
 
-    header = { 'T': Ttemplate, 'Tspec': Tspec, 'Tx' : Ttemplate, 'version':sabnzbd.__version__, 'paused': Downloader.do.paused,
+    header = { 'T': Ttemplate, 'Tspec': Tspec, 'Tx' : Ttemplate, 'version':sabnzbd.__version__,
+               'paused': Downloader.do.paused or Downloader.do.postproc,
                'pause_int': scheduler.pause_int(), 'paused_all': sabnzbd.PAUSED_ALL,
                'uptime':uptime, 'color_scheme':color }
     speed_limit = Downloader.do.get_limit()
@@ -1585,15 +1605,16 @@ def build_header(prim, webdir=''):
     header['quota'] = to_units(BPSMeter.do.quota)
     header['have_quota'] = bool(BPSMeter.do.quota > 0.0)
     header['left_quota'] = to_units(BPSMeter.do.left)
+    header['pp_pause_event'] = sabnzbd.scheduler.pp_pause_event()
 
     status = ''
-    if Downloader.do.paused:
+    if Downloader.do.paused or Downloader.do.postproc:
         status = Status.PAUSED
     elif bytespersec > 0:
         status = Status.DOWNLOADING
     else:
         status = 'Idle'
-    header['status'] = "%s" % status
+    header['status'] = status
 
     anfo  = ArticleCache.do.cache_info()
 
@@ -1673,9 +1694,11 @@ def build_history(start=None, limit=None, verbose=False, verbose_list=None, sear
     # Aquire the db instance
     try:
         history_db = cherrypy.thread_data.history_db
+        close_db = False
     except:
         # Required for repairs at startup because Cherrypy isn't active yet
         history_db = get_history_handle()
+        close_db = True
 
     # Fetch history items
     if not h_limit:
@@ -1741,6 +1764,9 @@ def build_history(start=None, limit=None, verbose=False, verbose_list=None, sear
     total_items += full_queue_size
     fetched_items = len(items)
 
+    if close_db:
+        history_db.close()
+
     return (items, fetched_items, total_items)
 
 
@@ -1932,3 +1958,13 @@ def del_from_section(kwargs):
     else:
         return False
 
+
+#------------------------------------------------------------------------------
+def history_remove_failed():
+    """ Remove all failed jobs from history, including files """
+    logging.info('Scheduled removal of all failed jobs')
+    history_db = get_history_handle()
+    del_job_files(history_db.get_failed_paths())
+    history_db.remove_failed()
+    history_db.close()
+    del history_db
