@@ -88,13 +88,17 @@ class Assembler(Thread):
                     try:
                         filepath = _assemble(nzf, filepath, dupe)
                     except IOError, (errno, strerror):
-                        # 28 == disk full => pause downloader
-                        if errno == 28:
-                            logging.error(Ta('Disk full! Forcing Pause'))
+                        if nzo.deleted:
+                            # Job was deleted, ignore error
+                            pass
                         else:
-                            logging.error(Ta('Disk error on creating file %s'), latin1(filepath))
-                        # Pause without saving
-                        sabnzbd.downloader.Downloader.do.pause(save=False)
+                            # 28 == disk full => pause downloader
+                            if errno == 28:
+                                logging.error(Ta('Disk full! Forcing Pause'))
+                            else:
+                                logging.error(Ta('Disk error on creating file %s'), latin1(filepath))
+                            # Pause without saving
+                            sabnzbd.downloader.Downloader.do.pause(save=False)
                     except:
                         logging.error('Fatal error in Assembler', exc_info = True)
                         break
@@ -108,8 +112,15 @@ class Assembler(Thread):
                             logging.debug('Got md5pack for set %s', setname)
 
                     if check_encrypted_rar(nzo, filepath):
-                        logging.warning(Ta('WARNING: Paused job "%s" because of encrypted RAR file'), latin1(nzo.final_name))
-                        nzo.pause()
+                        if cfg.pause_on_pwrar() == 1:
+                            logging.warning(Ta('WARNING: Paused job "%s" because of encrypted RAR file'), latin1(nzo.final_name))
+                            nzo.pause()
+                        else:
+                            logging.warning(Ta('WARNING: Aborted job "%s" because of encrypted RAR file'), latin1(nzo.final_name))
+                            nzo.fail_msg = T('Aborted, encryption detected')
+                            import sabnzbd.nzbqueue
+                            sabnzbd.nzbqueue.NzbQueue.do.end_job(nzo)
+                    nzf.completed = True
             else:
                 sabnzbd.nzbqueue.NzbQueue.do.remove(nzo.nzo_id, add_to_history=False, cleanup=False)
                 PostProcessor.do.process(nzo)
@@ -134,7 +145,7 @@ def _assemble(nzf, path, dupe):
     decodetable = nzf.decodetable
 
     for articlenum in decodetable:
-        sleep(0.01)
+        sleep(0.001)
         article = decodetable[articlenum]
 
         data = ArticleCache.do.load_article(article)
@@ -201,18 +212,19 @@ def file_has_articles(nzf):
 # For a full description of the par2 specification, visit:
 # http://parchive.sourceforge.net/docs/specifications/parity-volume-spec/article-spec.html
 
-def GetMD5Hashes(fname):
+def GetMD5Hashes(fname, force=False):
     """ Get the hash table from a PAR2 file
         Return as dictionary, indexed on names and True for utf8-encoded names
     """
-    new_encoding = False
+    new_encoding = True
     table = {}
-    if not flag_file(os.path.split(fname)[0], QCHECK_FILE):
+    if force or not flag_file(os.path.split(fname)[0], QCHECK_FILE):
         try:
             f = open(fname, 'rb')
         except:
             return table, new_encoding
 
+        new_encoding = False
         try:
             header = f.read(8)
             while header:
@@ -278,9 +290,16 @@ def ParseFilePacket(f, header):
 def is_cloaked(path, names):
     """ Return True if this is likely to be a cloaked encrypted post """
     fname = unicoder(os.path.split(path)[1]).lower()
+    fname = os.path.splitext(fname)[0]
     for name in names:
-        name = unicoder(name.lower())
-        if fname == name or 'password' in name:
+        name = os.path.split(name.lower())[1]
+        name, ext = os.path.splitext(unicoder(name))
+        if ext == u'.rar' and fname.startswith(name) and (len(fname) - len(name)) < 8 and \
+           '.subs' not in fname:
+            logging.debug('File %s is probably encrypted due to RAR with same name inside this RAR', fname)
+            return True
+        elif 'password' in name:
+            logging.debug('RAR %s is probably encrypted: "password" in filename %s', fname, name)
             return True
     return False
 
@@ -288,11 +307,11 @@ def is_cloaked(path, names):
 def check_encrypted_rar(nzo, filepath):
     """ Check if file is rar and is encrypted """
     encrypted = False
-    if not nzo.password and cfg.pause_on_pwrar() and is_rarfile(filepath):
+    if  not nzo.password and not nzo.meta.get('password') and cfg.pause_on_pwrar() and is_rarfile(filepath):
         try:
             zf = RarFile(filepath, all_names=True)
             encrypted = zf.encrypted or is_cloaked(filepath, zf.namelist())
-            if encrypted and int(nzo.encrypted) < 2:
+            if encrypted and int(nzo.encrypted) < 2 and not nzo.reuse:
                 nzo.encrypted = 1
             else:
                 encrypted = False
