@@ -139,7 +139,7 @@ class OptionNumber(Option):
                 else:
                     value = float(value)
             except ValueError:
-                value = 0
+                value = self._Option__default_val
             if self.__validation:
                 error, val = self.__validation(value)
                 self._Option__set(val)
@@ -186,6 +186,14 @@ class OptionDir(Option):
                 res, path = sabnzbd.misc.create_real_path(self.ident()[1], self.__root, value, self.__apply_umask)
         return path
 
+    def test_path(self):
+        """ Return True if path exists """
+        value = self.get()
+        if value:
+            return os.path.exists(sabnzbd.misc.real_path(self.__root, value))
+        else:
+            return False
+
     def set_root(self, root):
         """ Set new root, is assumed to be valid """
         self.__root = root
@@ -210,6 +218,10 @@ class OptionDir(Option):
                 self._Option__set(value)
         return error
 
+    def set_create(self, value):
+        """ Set auto-creation value """
+        self.__create = value
+
 
 class OptionList(Option):
     """ List option class """
@@ -224,7 +236,10 @@ class OptionList(Option):
         error = None
         if value is not None:
             if not isinstance(value, list):
-                value = listquote.simplelist(value)
+                if '"' not in value and ',' not in value:
+                    value = value.split()
+                else:
+                    value = listquote.simplelist(value)
             if self.__validation:
                 error, value = self.__validation(value)
             if not error:
