@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -38,6 +38,7 @@ import sabnzbd.cfg
 from sabnzbd.constants import DB_HISTORY_NAME
 from sabnzbd.encoding import unicoder
 from sabnzbd.bpsmeter import this_week, this_month
+from sabnzbd.misc import format_source_url
 
 _HISTORY_DB = None        # Will contain full path to history database
 _DONE_CLEANING = False    # Ensure we only do one Vacuum per session
@@ -50,6 +51,27 @@ def get_history_handle():
     return HistoryDB(_HISTORY_DB)
 
 
+def convert_search(search):
+    """ Convert classic wildcard to SQL wildcard """
+    if not search:
+        # Default value
+        search = ''
+    else:
+        # Allow * for wildcard matching and space
+        search = search.replace('*','%').replace(' ', '%')
+
+    # Allow ^ for start of string and $ for end of string
+    if search and search.startswith('^'):
+        search = search.replace('^','')
+        search += '%'
+    elif search and search.endswith('$'):
+        search = search.replace('$','')
+        search = '%' + search
+    else:
+        search = '%' + search + '%'
+    return search
+
+
 # Note: Add support for execute return values
 
 class HistoryDB(object):
@@ -60,7 +82,7 @@ class HistoryDB(object):
             create_table = True
         else:
             create_table = False
-        if sabnzbd.WIN32:
+        if sabnzbd.WIN32 and isinstance(db_path, str):
             self.con = sqlite3.connect(db_path.decode('latin-1').encode('utf-8'))
         else:
             self.con = sqlite3.connect(db_path)
@@ -140,25 +162,28 @@ class HistoryDB(object):
             logging.error(Ta('Failed to close database, see log'))
             logging.info("Traceback: ", exc_info = True)
 
-    def remove_completed(self):
-        return self.execute("""DELETE FROM history WHERE status = 'Completed'""", save=True)
+    def remove_completed(self, search=None):
+        search = convert_search(search)
+        return self.execute("""DELETE FROM history WHERE name LIKE ? AND status = 'Completed'""", (search,), save=True)
 
-    def get_failed_paths(self):
+    def get_failed_paths(self, search=None):
         """ Return list of all storage paths of failed jobs (may contain non-existing or empty paths) """
-        fetch_ok = self.execute("""SELECT path FROM history WHERE status = 'Failed'""")
+        search = convert_search(search)
+        fetch_ok = self.execute("""SELECT path FROM history WHERE name LIKE ? AND status = 'Failed'""", (search,))
         if fetch_ok:
             return [item.get('path') for item in self.c.fetchall()]
         else:
             return []
 
-    def remove_failed(self):
-        return self.execute("""DELETE FROM history WHERE status = 'Failed'""", save=True)
+    def remove_failed(self, search=None):
+        search = convert_search(search)
+        return self.execute("""DELETE FROM history WHERE name LIKE ? AND status = 'Failed'""", (search,), save=True)
 
     def remove_history(self, jobs=None):
         if jobs is None:
             self.remove_completed()
         else:
-            if type(jobs) == type(''):
+            if not isinstance(jobs, list):
                 jobs = [jobs]
 
             for job in jobs:
@@ -179,32 +204,19 @@ class HistoryDB(object):
 
     def fetch_history(self, start=None, limit=None, search=None, failed_only=0):
 
-        if not search:
-            # Default value
-            search = ''
-        else:
-            # Allow * for wildcard matching and space
-            search = search.replace('*','%').replace(' ', '%')
-
-        # Allow ^ for start of string and $ for end of string
-        if search and search.startswith('^'):
-            search = search.replace('^','')
-            search += '%'
-        elif search and search.endswith('$'):
-            search = search.replace('$','')
-            search = '%' + search
-        else:
-            search = '%' + search + '%'
+        search = convert_search(search)
 
         # Get the number of results
         if failed_only:
             res = self.execute('select count(*) from History WHERE name LIKE ? AND STATUS = "Failed"', (search,))
         else:
             res = self.execute('select count(*) from History WHERE name LIKE ?', (search,))
+        total_items = -1
         if res:
-            total_items = self.c.fetchone().get('count(*)')
-        else:
-            total_items = -1
+            try:
+                total_items = self.c.fetchone().get('count(*)')
+            except AttributeError:
+                pass
 
         if not start:
             start = 0
@@ -236,31 +248,34 @@ class HistoryDB(object):
         amounts downloaded in the last month and week
         """
         # Total Size of the history
+        total = 0
         if self.execute('''SELECT sum(bytes) FROM history'''):
-            f = self.c.fetchone()
-            total = f.get('sum(bytes)')
-        else:
-            total = 0
+            try:
+                total = self.c.fetchone().get('sum(bytes)')
+            except AttributeError:
+                pass
 
         # Amount downloaded this month
         #r = time.gmtime(time.time())
         #month_timest = int(time.mktime((r.tm_year, r.tm_mon, 0, 0, 0, 1, r.tm_wday, r.tm_yday, r.tm_isdst)))
         month_timest = int(this_month(time.time()))
 
+        month = 0
         if self.execute('''SELECT sum(bytes) FROM history WHERE "completed">?''', (month_timest,)):
-            f = self.c.fetchone()
-            month = f.get('sum(bytes)')
-        else:
-            month = 0
+            try:
+                month = self.c.fetchone().get('sum(bytes)')
+            except AttributeError:
+                pass
 
         # Amount downloaded this week
         week_timest = int(this_week(time.time()))
 
+        week = 0
         if self.execute('''SELECT sum(bytes) FROM history WHERE "completed">?''', (week_timest,)):
-            f = self.c.fetchone()
-            week = f.get('sum(bytes)')
-        else:
-            week = 0
+            try:
+                week = self.c.fetchone().get('sum(bytes)')
+            except AttributeError:
+                pass
 
         return (total, month, week)
 
@@ -269,27 +284,32 @@ class HistoryDB(object):
         data = ''
         t = (nzo_id,)
         if self.execute('SELECT script_log FROM history WHERE nzo_id=?', t):
-            f = self.c.fetchone()
             try:
-                data = zlib.decompress(f.get('script_log'))
+                data = zlib.decompress(self.c.fetchone().get('script_log'))
             except:
-                data = ''
+                pass
         return data
 
     def get_name(self, nzo_id):
         t = (nzo_id,)
+        name = ''
         if self.execute('SELECT name FROM history WHERE nzo_id=?', t):
-            return self.c.fetchone().get('name')
-        else:
-            return ''
+            try:
+                name = self.c.fetchone().get('name')
+            except AttributeError:
+                pass
+        return name
 
 
     def get_path(self, nzo_id):
         t = (nzo_id,)
+        path = ''
         if self.execute('SELECT path FROM history WHERE nzo_id=?', t):
-            return self.c.fetchone().get('path')
-        else:
-            return ''
+            try:
+                path = self.c.fetchone().get('path')
+            except AttributeError:
+                pass
+        return path
 
 def dict_factory(cursor, row):
     d = {}
@@ -312,9 +332,9 @@ def build_history_info(nzo, storage='', downpath='', postproc_time=0, script_out
     # Get the url and newzbin msgid
     report = decode_factory(nzo_info.get('msgid', ''))
     if report:
-        url = 'https://%s/browse/post/%s/' % (sabnzbd.cfg.newzbin_url(), report)
+        url = format_source_url(report)
     else:
-        url = decode_factory(nzo_info.get('url', ''))
+        url = decode_factory(nzo.url)
 
     #group = nzo.group
 
