@@ -23,6 +23,7 @@ import Queue
 import binascii
 import logging
 import re
+from time import sleep
 from threading import Thread
 try:
     import _yenc
@@ -72,6 +73,7 @@ class Decoder(Thread):
     def run(self):
         from sabnzbd.nzbqueue import NzbQueue
         while 1:
+            sleep(0.001)
             art_tup = self.queue.get()
             if not art_tup:
                 break
@@ -82,6 +84,8 @@ class Decoder(Thread):
             article, lines = art_tup
             nzf = article.nzf
             nzo = nzf.nzo
+            art_id = article.article
+            killed = False
 
             data = None
 
@@ -94,14 +98,16 @@ class Decoder(Thread):
                     if nzo.precheck:
                         raise BadYenc
                     register = True
-                    logging.debug("Decoding %s", article)
+                    logging.debug("Decoding %s", art_id)
 
                     data = decode(article, lines)
                     nzf.article_count += 1
                     found = True
                 except IOError, e:
-                    logme = Ta('Decoding %s failed') % article
-                    logging.info(logme)
+                    logme = Ta('Decoding %s failed') % art_id
+                    logging.warning(logme)
+                    logging.info("Traceback: ", exc_info = True)
+
                     sabnzbd.downloader.Downloader.do.pause()
 
                     article.fetcher = None
@@ -111,7 +117,7 @@ class Decoder(Thread):
                     register = False
 
                 except CrcError, e:
-                    logme = Ta('CRC Error in %s (%s -> %s)') % (article, e.needcrc, e.gotcrc)
+                    logme = Ta('CRC Error in %s (%s -> %s)') % (art_id, e.needcrc, e.gotcrc)
                     logging.info(logme)
 
                     data = e.data
@@ -133,27 +139,32 @@ class Decoder(Thread):
                         # Examine headers (for precheck) or body (for download)
                         # And look for DMCA clues (while skipping "X-" headers)
                         for line in lines:
-                            if not line.startswith('X-') and match_str(line.lower(), ('dmca', 'removed', 'cancel', 'blocked')):
-                                logging.info('Article removed from server (%s)', article)
+                            lline = line.lower()
+                            if 'message-id:' in lline:
+                                found = True
+                            if not line.startswith('X-') and match_str(lline, ('dmca', 'removed', 'cancel', 'blocked')):
                                 killed = True
                                 break
+                    if killed:
+                        logme = 'Article removed from server (%s)'
+                        logging.info(logme, art_id)
                     if nzo.precheck:
-                        if found or not killed:
+                        if found and not killed:
                             # Pre-check, proper article found, just register
-                            logging.debug('Server has article %s', article)
+                            logging.debug('Server has article %s', art_id)
                             register = True
                     elif not killed and not found:
-                        logme = Ta('Badly formed yEnc article in %s') % article
+                        logme = Ta('Badly formed yEnc article in %s') % art_id
                         logging.info(logme)
 
-                    if not found:
+                    if not found or killed:
                         new_server_found = self.__search_new_server(article)
                         if new_server_found:
                             register = False
                             logme = None
 
                 except:
-                    logme = Ta('Unknown Error while decoding %s') % article
+                    logme = Ta('Unknown Error while decoding %s') % art_id
                     logging.info(logme)
                     logging.info("Traceback: ", exc_info = True)
 
@@ -163,7 +174,10 @@ class Decoder(Thread):
                         logme = None
 
                 if logme:
-                    article.nzf.nzo.inc_log('bad_art_log', logme)
+                    if killed:
+                        article.nzf.nzo.inc_log('killed_art_log', art_id)
+                    else:
+                        article.nzf.nzo.inc_log('bad_art_log', art_id)
 
             else:
                 new_server_found = self.__search_new_server(article)
@@ -234,7 +248,7 @@ def decode(article, data):
         if not ybegin:
             found = False
             try:
-                for i in xrange(10):
+                for i in xrange(min(40, len(data))):
                     if data[i].startswith('begin '):
                         nzf.filename = name_fixer(data[i].split(None, 2)[2])
                         nzf.type = 'uu'
@@ -250,7 +264,10 @@ def decode(article, data):
             except IndexError:
                 raise BadYenc()
 
-            decoded_data = '\r\n'.join(data)
+            if found:
+                decoded_data = '\r\n'.join(data)
+            else:
+                raise BadYenc()
 
         #Deal with yenc encoded posts
         elif (ybegin and yend):
@@ -298,7 +315,7 @@ def yCheck(data):
     yend = None
 
     ## Check head
-    for i in xrange(40):
+    for i in xrange(min(40, len(data))):
         try:
             if data[i].startswith('=ybegin '):
                 splits = 3
