@@ -47,6 +47,8 @@ _PENALTY_SHARE   = 10   # Account sharing detected
 _PENALTY_TOOMANY = 10   # Too many connections
 _PENALTY_PERM    = 10   # Permanent error, like bad username/password
 _PENALTY_SHORT   = 1    # Minimal penalty when no_penalties is set
+_PENALTY_VERYSHORT = 0.1 # Error 400 without cause clues
+
 
 TIMER_LOCK = RLock()
 
@@ -77,7 +79,8 @@ class Server(object):
         self.warning = ''
         self.info = None     # Will hold getaddrinfo() list
         self.request = False # True if a getaddrinfo() request is pending
-        self.oddball = 'free.xsusenet.com' in host
+        self.have_body = 'free.xsusenet.com' not in host
+        self.have_stat = True # Assume server has "STAT", until proven otherwise
 
         for i in range(threads):
             self.idle_threads.append(NewsWrapper(self, i+1))
@@ -502,8 +505,7 @@ class Downloader(Thread):
                             ecode = msg[:3]
                             display_msg = ' [%s]' % msg
                             logging.debug('Server login problem: %s, %s', ecode, msg)
-                            if ((ecode in ('502', '400')) and clues_too_many(msg)) or \
-                                (ecode == '481' and clues_too_many(msg)):
+                            if ecode in ('502', '481', '400') and clues_too_many(msg):
                                 # Too many connections: remove this thread and reduce thread-setting for server
                                 # Plan to go back to the full number after a penalty timeout
                                 if server.active:
@@ -536,6 +538,12 @@ class Downloader(Thread):
                                 else:
                                     penalty = _PENALTY_502
                                 block = True
+                            elif ecode == '400':
+                                # Temp connection problem?
+                                if server.active:
+                                    logging.debug('Unspecified error 400 from server %s', server.host)
+                                penalty = _PENALTY_VERYSHORT
+                                block = True
                             else:
                                 # Unknown error, just keep trying
                                 if server.active:
@@ -599,6 +607,19 @@ class Downloader(Thread):
                         msg = T('Server %s requires user/password') % ('%s:%s' % (nw.server.host, nw.server.port))
                         self.__reset_nw(nw, msg, quit=True)
 
+                    elif code == '500':
+                        if nzo.precheck:
+                            # Assume "STAT" command is not supported
+                            server.have_stat = False
+                            logging.debug('Server %s does not support STAT', server.host)
+                        else:
+                            # Assume "BODY" command is not supported
+                            server.have_body = False
+                            logging.debug('Server %s does not support BODY', server.host)
+                        nw.lines = []
+                        nw.data = ''
+                        self.__request_article(nw)
+
                 if done:
                     server.bad_cons = 0 # Succesful data, clear "bad" counter
                     if sabnzbd.LOG_ALL:
