@@ -84,6 +84,12 @@ def check_server(host, port):
         return badParameterResponse(T('Server address "%s:%s" is not valid.') % (host, port))
 
 
+def check_access():
+    """ Check if external address is allowed """
+    referrer = cherrypy.request.remote.ip
+    return referrer in ('127.0.0.1', '::1') or referrer.startswith(cfg.local_range())
+
+
 def ConvertSpecials(p):
     """ Convert None to 'None' and 'Default' to ''
     """
@@ -158,6 +164,8 @@ def set_auth(conf):
 
 def check_session(kwargs):
     """ Check session key """
+    if not check_access():
+        return u'No access'
     key = kwargs.get('session')
     if not key:
         key = kwargs.get('apikey')
@@ -176,6 +184,10 @@ def check_apikey(kwargs, nokey=False):
     """ Check api key or nzbkey
         Return None when OK, otherwise an error message
     """
+    def log_warning(txt):
+        txt = '%s %s>%s' % (txt, cherrypy.request.remote.ip, cherrypy.request.headers.get('User-Agent', '??'))
+        logging.warning('%s', txt)
+
     output = kwargs.get('output')
     mode = kwargs.get('mode', '')
     callback = kwargs.get('callback')
@@ -188,19 +200,22 @@ def check_apikey(kwargs, nokey=False):
     # For NZB upload calls, a separate key can be used
     nzbkey = kwargs.get('mode', '') in ('addid', 'addurl', 'addfile', 'addlocalfile')
 
+    if not nzbkey and not check_access():
+        return report(output, 'No access')
+
     # First check APIKEY, if OK that's sufficient
     if not (cfg.disable_key() or nokey):
         key = kwargs.get('apikey')
         if not key:
             if not special:
-                logging.warning(Ta('API Key missing, please enter the api key from Config->General into your 3rd party program:'))
+                log_warning(Ta('API Key missing, please enter the api key from Config->General into your 3rd party program:'))
             return report(output, 'API Key Required', callback=callback)
         elif nzbkey and key == cfg.nzb_key():
             return None
         elif key == cfg.api_key():
             return None
         else:
-            logging.warning(Ta('API Key incorrect, Use the api key from Config->General in your 3rd party program:'))
+            log_warning(Ta('API Key incorrect, Use the api key from Config->General in your 3rd party program:'))
             return report(output, 'API Key Incorrect', callback=callback)
 
     # No active APIKEY, check web credentials instead
@@ -209,7 +224,7 @@ def check_apikey(kwargs, nokey=False):
             pass
         else:
             if not special:
-                logging.warning(Ta('Authentication missing, please enter username/password from Config->General into your 3rd party program:'))
+                log_warning(Ta('Authentication missing, please enter username/password from Config->General into your 3rd party program:'))
             return report(output, 'Missing authentication', callback=callback)
     return None
 
@@ -249,6 +264,8 @@ class MainPage(object):
 
     @cherrypy.expose
     def index(self, **kwargs):
+        if not check_access(): return Protected()
+
         if sabnzbd.OLD_QUEUE and not cfg.warned_old_queue():
             cfg.warned_old_queue.set(True)
             config.save_config()
@@ -293,6 +310,7 @@ class MainPage(object):
 
 
     def add_handler(self, kwargs):
+        if not check_access(): return Protected()
         id = kwargs.get('id', '')
         if not id:
             id = kwargs.get('url', '')
@@ -397,7 +415,9 @@ class MainPage(object):
     def api(self, **kwargs):
         """Handler for API over http, with explicit authentication parameters
         """
-        logging.debug('API-call from %s %s', cherrypy.request.remote.ip, kwargs)
+        if not kwargs.get('tickleme') or not cfg.web_watchdog():
+            logging.debug('API-call from %s [%s] %s', cherrypy.request.remote.ip, \
+                          cherrypy.request.headers.get('User-Agent', '??'), kwargs)
         if kwargs.get('mode', '') not in ('version', 'auth'):
             msg = check_apikey(kwargs)
             if msg: return msg
@@ -407,6 +427,7 @@ class MainPage(object):
     def scriptlog(self, **kwargs):
         """ Duplicate of scriptlog of History, needed for some skins """
         # No session key check, due to fixed URLs
+        if not check_access(): return Protected()
 
         name = kwargs.get('name')
         if name:
@@ -440,6 +461,12 @@ class MainPage(object):
         retry_job(kwargs.get('job'), kwargs.get('nzbfile'))
         raise dcRaiser(self.__root, kwargs)
 
+    @cherrypy.expose
+    def robots_txt(self):
+        """ Keep web crawlers out """
+        cherrypy.response.headers['Content-Type'] = 'text/plain'
+        return 'User-agent: *\nDisallow: /\n'
+
 
 #------------------------------------------------------------------------------
 class NzoPage(object):
@@ -458,7 +485,7 @@ class NzoPage(object):
         # /nzb/SABnzbd_nzo_xxxxx/files
         # /nzb/SABnzbd_nzo_xxxxx/bulk_operation
         # /nzb/SABnzbd_nzo_xxxxx/save
-
+        if not check_access(): return Protected()
         nzo_id = None
         for a in args:
             if a.startswith('SABnzbd_nzo'):
@@ -513,12 +540,19 @@ class NzoPage(object):
                 cat = pnfo[PNFO_EXTRA_FIELD1]
                 if not cat:
                     cat = 'None'
-                filename = xml_name(nzo.final_name_pw_clean)
+                filename_pw = xml_name(nzo.final_name_pw_clean)
+                filename = xml_name(nzo.final_name)
+                if nzo.password:
+                    password = xml_name(nzo.password)
+                else:
+                    password = ''
                 priority = pnfo[PNFO_PRIORITY_FIELD]
 
                 slot['nzo_id'] =  str(nzo_id)
                 slot['cat'] = cat
-                slot['filename'] = filename
+                slot['filename'] = filename_pw
+                slot['filename_clean'] = filename
+                slot['password'] = password or ''
                 slot['script'] = script
                 slot['priority'] = str(priority)
                 slot['unpackopts'] = str(unpackopts)
@@ -566,6 +600,7 @@ class NzoPage(object):
     def save_details(self, nzo_id, args, kwargs):
         index = kwargs.get('index', None)
         name = kwargs.get('name', None)
+        password = kwargs.get('password', None)
         pp = kwargs.get('pp', None)
         script = kwargs.get('script', None)
         cat = kwargs.get('cat', None)
@@ -575,7 +610,7 @@ class NzoPage(object):
         if index != None:
             NzbQueue.do.switch(nzo_id, index)
         if name != None:
-            NzbQueue.do.change_name(nzo_id, special_fixer(name))
+            NzbQueue.do.change_name(nzo_id, special_fixer(name), password)
         if cat != None:
             NzbQueue.do.change_cat(nzo_id,cat)
         if script != None:
@@ -629,6 +664,7 @@ class QueuePage(object):
 
     @cherrypy.expose
     def index(self, **kwargs):
+        if not check_access(): return Protected()
         start = kwargs.get('start')
         limit = kwargs.get('limit')
         dummy2 = kwargs.get('dummy2')
@@ -845,6 +881,7 @@ class HistoryPage(object):
 
     @cherrypy.expose
     def index(self, **kwargs):
+        if not check_access(): return Protected()
         start = kwargs.get('start')
         limit = kwargs.get('limit')
         search = kwargs.get('search')
@@ -863,9 +900,11 @@ class HistoryPage(object):
         #history_items, total_bytes, bytes_beginning = sabnzbd.history_info()
         #history['bytes_beginning'] = "%.2f" % (bytes_beginning / GIGI)
 
+        postfix = T('B') #: Abbreviation for bytes, as in GB
         grand, month, week, day = BPSMeter.do.get_sums()
         history['total_size'], history['month_size'], history['week_size'], history['day_size'] = \
-               to_units(grand), to_units(month), to_units(week), to_units(day)
+               to_units(grand, postfix=postfix), to_units(month, postfix=postfix), \
+               to_units(week, postfix=postfix), to_units(day, postfix=postfix)
 
         history['lines'], history['fetched'], history['noofslots'] = build_history(limit=limit, start=start, verbose=self.__verbose, verbose_list=self.__verbose_list, search=search, failed_only=failed_only)
 
@@ -963,7 +1002,7 @@ class HistoryPage(object):
     def scriptlog(self, **kwargs):
         """ Duplicate of scriptlog of History, needed for some skins """
         # No session key check, due to fixed URLs
-
+        if not check_access(): return Protected()
         name = kwargs.get('name')
         if name:
             history_db = cherrypy.thread_data.history_db
@@ -1009,6 +1048,7 @@ class ConfigPage(object):
 
     @cherrypy.expose
     def index(self, **kwargs):
+        if not check_access(): return Protected()
         conf, pnfo_list, bytespersec = build_header(self.__prim, self.__web_dir)
 
         conf['configfn'] = config.get_filename()
@@ -1065,12 +1105,14 @@ class ConfigPage(object):
 def orphan_delete(kwargs):
     path = kwargs.get('name')
     if path:
+        path = platform_encode(path)
         path = os.path.join(cfg.download_dir.get_path(), path)
         remove_all(path, recursive=True)
 
 def orphan_add(kwargs):
     path = kwargs.get('name')
     if path:
+        path = platform_encode(path)
         path = os.path.join(cfg.download_dir.get_path(), path)
         sabnzbd.nzbqueue.repair_job(path, None)
 
@@ -1090,7 +1132,7 @@ class ConfigFolders(object):
 
     @cherrypy.expose
     def index(self, **kwargs):
-        if cfg.configlock():
+        if cfg.configlock() or not check_access():
             return Protected()
 
         conf, pnfo_list, bytespersec = build_header(self.__prim, self.__web_dir)
@@ -1134,7 +1176,7 @@ SWITCH_LIST = \
              'ignore_samples', 'pause_on_post_processing', 'quick_check', 'nice', 'ionice',
              'ssl_type', 'pre_script', 'pause_on_pwrar', 'ampm', 'sfv_check', 'folder_rename',
              'unpack_check', 'quota_size', 'quota_day', 'quota_resume', 'quota_period',
-             'pre_check', 'max_art_tries', 'max_art_opt'
+             'pre_check', 'max_art_tries', 'max_art_opt', 'fail_hopeless'
              )
 
 #------------------------------------------------------------------------------
@@ -1146,7 +1188,7 @@ class ConfigSwitches(object):
 
     @cherrypy.expose
     def index(self, **kwargs):
-        if cfg.configlock():
+        if cfg.configlock() or not check_access():
             return Protected()
 
         conf, pnfo_list, bytespersec = build_header(self.__prim, self.__web_dir)
@@ -1188,12 +1230,17 @@ SPECIAL_BOOL_LIST = \
               'queue_complete_pers', 'api_warnings', 'allow_64bit_tools', 'par2_multicore',
               'never_repair', 'allow_streaming', 'ignore_unrar_dates', 'rss_filenames',
               'osx_menu', 'osx_speed', 'win_menu', 'uniconfig', 'use_pickle', 'allow_incomplete_nzb',
-              'randomize_server_ip', 'no_ipv6'
+              'randomize_server_ip', 'no_ipv6', 'keep_awake', 'overwrite_files', 'empty_postproc',
+              'web_watchdog', 'wait_for_dfolder', 'warn_empty_nzb', 'enable_recursive', 'sanitize_safe'
+
             )
 SPECIAL_VALUE_LIST = \
             ( 'size_limit', 'folder_max_length', 'fsys_type', 'movie_rename_limit', 'nomedia_marker',
-              'req_completion_rate', 'wait_ext_drive'
+              'req_completion_rate', 'wait_ext_drive', 'history_limit', 'show_sysload', 'ipv6_servers'
             )
+SPECIAL_LIST_LIST = \
+    ( 'rss_odd_titles', 'prio_sort_list'
+    )
 
 class ConfigSpecial(object):
     def __init__(self, web_dir, root, prim):
@@ -1203,7 +1250,7 @@ class ConfigSpecial(object):
 
     @cherrypy.expose
     def index(self, **kwargs):
-        if cfg.configlock():
+        if cfg.configlock() or not check_access():
             return Protected()
 
         conf, pnfo_list, bytespersec = build_header(self.__prim, self.__web_dir)
@@ -1212,6 +1259,7 @@ class ConfigSpecial(object):
 
         conf['switches'] = [ (kw, config.get_config('misc', kw)(), config.get_config('misc', kw).default()) for kw in SPECIAL_BOOL_LIST]
         conf['entries'] = [ (kw, config.get_config('misc', kw)(), config.get_config('misc', kw).default()) for kw in SPECIAL_VALUE_LIST]
+        conf['entries'].extend( [ (kw, config.get_config('misc', kw).get_string(), '') for kw in SPECIAL_LIST_LIST] )
 
         template = Template(file=os.path.join(self.__web_dir, 'config_special.tmpl'),
                             filter=FILTER, searchList=[conf], compilerSettings=DIRECTIVES)
@@ -1222,7 +1270,7 @@ class ConfigSpecial(object):
         msg = check_session(kwargs)
         if msg: return msg
 
-        for kw in SPECIAL_BOOL_LIST + SPECIAL_VALUE_LIST:
+        for kw in SPECIAL_BOOL_LIST + SPECIAL_VALUE_LIST + SPECIAL_LIST_LIST:
             item = config.get_config('misc', kw)
             value = kwargs.get(kw)
             msg = item.set(value)
@@ -1236,8 +1284,8 @@ class ConfigSpecial(object):
 #------------------------------------------------------------------------------
 GENERAL_LIST = (
     'host', 'port', 'username', 'password', 'disable_api_key',
-    'refresh_rate', 'cache_limit',
-    'enable_https', 'https_port', 'https_cert', 'https_key'
+    'refresh_rate', 'cache_limit', 'local_range',
+    'enable_https', 'https_port', 'https_cert', 'https_key', 'https_chain'
 )
 
 class ConfigGeneral(object):
@@ -1271,7 +1319,7 @@ class ConfigGeneral(object):
             else:
                 return ''
 
-        if cfg.configlock():
+        if cfg.configlock() or not check_access():
             return Protected()
 
         conf, pnfo_list, bytespersec = build_header(self.__prim, self.__web_dir)
@@ -1335,6 +1383,7 @@ class ConfigGeneral(object):
         conf['https_port'] = cfg.https_port()
         conf['https_cert'] = cfg.https_cert()
         conf['https_key'] = cfg.https_key()
+        conf['https_chain'] = cfg.https_chain()
         conf['enable_https'] = cfg.enable_https()
         conf['username'] = cfg.username()
         conf['password'] = cfg.password.get_stars()
@@ -1343,6 +1392,8 @@ class ConfigGeneral(object):
         conf['cache_limit'] = cfg.cache_limit()
         conf['cleanup_list'] = cfg.cleanup_list.get_string()
         conf['nzb_key'] = cfg.nzb_key()
+        conf['local_range'] = cfg.local_range()
+        conf['my_lcldata'] = cfg.admin_dir.get_path()
 
         template = Template(file=os.path.join(self.__web_dir, 'config_general.tmpl'),
                             filter=FILTER, searchList=[conf], compilerSettings=DIRECTIVES)
@@ -1450,7 +1501,7 @@ class ConfigServer(object):
 
     @cherrypy.expose
     def index(self, **kwargs):
-        if cfg.configlock():
+        if cfg.configlock() or not check_access():
             return Protected()
 
         conf, pnfo_list, bytespersec = build_header(self.__prim, self.__web_dir)
@@ -1609,7 +1660,7 @@ class ConfigRss(object):
 
     @cherrypy.expose
     def index(self, **kwargs):
-        if cfg.configlock():
+        if cfg.configlock() or not check_access():
             return Protected()
 
         conf, pnfo_list, bytespersec = build_header(self.__prim, self.__web_dir)
@@ -1639,6 +1690,7 @@ class ConfigRss(object):
         active_feed = kwargs.get('feed', '')
         conf['active_feed'] = active_feed
         conf['rss'] = rss
+        conf['rss_next'] = time.strftime(time_format('%H:%M'),time.localtime(sabnzbd.rss.next_run()))
 
         if active_feed:
             readout = bool(self.__refresh_readout)
@@ -1885,7 +1937,7 @@ class ConfigRss(object):
 
 #------------------------------------------------------------------------------
 _SCHED_ACTIONS = ('resume', 'pause', 'pause_all', 'shutdown', 'restart', 'speedlimit',
-                  'pause_post', 'resume_post', 'scan_folder', 'rss_scan')
+                  'pause_post', 'resume_post', 'scan_folder', 'rss_scan', 'remove_failed')
 
 class ConfigScheduling(object):
     def __init__(self, web_dir, root, prim):
@@ -1907,22 +1959,22 @@ class ConfigScheduling(object):
             days["7"] = T('Sunday')
             return days
 
-        if cfg.configlock():
+        if cfg.configlock() or not check_access():
             return Protected()
 
         conf, pnfo_list, bytespersec = build_header(self.__prim, self.__web_dir)
 
         actions = []
         actions.extend(_SCHED_ACTIONS)
-        days = get_days()
+        day_names = get_days()
         conf['schedlines'] = []
         snum = 1
         conf['taskinfo'] = []
-        for ev in scheduler.sort_schedules(forward=True):
+        for ev in scheduler.sort_schedules(all_events=False):
             line = ev[3]
             conf['schedlines'].append(line)
             try:
-                m, h, day, action = line.split(' ', 3)
+                m, h, day_numbers, action = line.split(' ', 3)
             except:
                 continue
             action = action.strip()
@@ -1942,7 +1994,17 @@ class ConfigScheduling(object):
                     act = ''
                 if act in ('enable_server', 'disable_server'):
                     action = Ttemplate("sch-" + act) + ' ' + server
-            item = (snum, '%02d' % int(h), '%02d' % int(m), days.get(day, day), '%s %s' % (action, value))
+
+            if day_numbers == "1234567":
+                days_of_week = "Daily"
+            elif day_numbers == "12345":
+                days_of_week = "Weekdays"
+            elif day_numbers == "67":
+                days_of_week = "Weekends"
+            else:
+                days_of_week = ", ".join([day_names.get(i, "**") for i in day_numbers])
+            item = (snum, '%02d' % int(h), '%02d' % int(m), days_of_week, '%s %s' % (action, value))
+
             conf['taskinfo'].append(item)
             snum += 1
 
@@ -1967,7 +2029,9 @@ class ConfigScheduling(object):
 
         minute = kwargs.get('minute')
         hour = kwargs.get('hour')
-        dayofweek = kwargs.get('dayofweek')
+        days_of_week = ''.join([str(x) for x in kwargs.get('daysofweek', '')])
+        if not days_of_week:
+            days_of_week = '1234567'
         action = kwargs.get('action')
         arguments = kwargs.get('arguments')
 
@@ -1977,7 +2041,7 @@ class ConfigScheduling(object):
         elif arguments in ('off','disable'):
             arguments = '0'
 
-        if minute and hour  and dayofweek and action:
+        if minute and hour  and days_of_week and action:
             if action == 'speedlimit':
                 if not (arguments and arguments.isdigit()):
                     action = '0'
@@ -1996,7 +2060,7 @@ class ConfigScheduling(object):
             if action:
                 sched = cfg.schedules()
                 sched.append('%s %s %s %s %s' %
-                             (minute, hour, dayofweek, action, arguments))
+                             (minute, hour, days_of_week, action, arguments))
                 cfg.schedules.set(sched)
 
         config.save_config()
@@ -2027,7 +2091,7 @@ class ConfigIndexers(object):
 
     @cherrypy.expose
     def index(self, **kwargs):
-        if cfg.configlock():
+        if cfg.configlock() or not check_access():
             return Protected()
 
         conf, pnfo_list, bytespersec = build_header(self.__prim, self.__web_dir)
@@ -2110,7 +2174,7 @@ class ConfigCats(object):
 
     @cherrypy.expose
     def index(self, **kwargs):
-        if cfg.configlock():
+        if cfg.configlock() or not check_access():
             return Protected()
 
         conf, pnfo_list, bytespersec = build_header(self.__prim, self.__web_dir)
@@ -2184,7 +2248,7 @@ class ConfigSorting(object):
 
     @cherrypy.expose
     def index(self, **kwargs):
-        if cfg.configlock():
+        if cfg.configlock() or not check_access():
             return Protected()
 
         conf, pnfo_list, bytespersec = build_header(self.__prim, self.__web_dir)
@@ -2237,6 +2301,7 @@ class Status(object):
 
     @cherrypy.expose
     def index(self, **kwargs):
+        if not check_access(): return Protected()
         header, pnfo_list, bytespersec = build_header(self.__prim, self.__web_dir)
 
         header['logfile'] = sabnzbd.LOGFILE
@@ -2488,10 +2553,15 @@ def GetRssLog(feed):
     # Sort in the order the jobs came from the feed
     names.sort(lambda x, y: jobs[x].get('order', 0) - jobs[y].get('order', 0))
 
-    done = [xml_name(jobs[job]['title']) for job in names if jobs[job]['status'] == 'D']
     good = [make_item(jobs[job]) for job in names if jobs[job]['status'][0] == 'G']
     bad  = [make_item(jobs[job]) for job in names if jobs[job]['status'][0] == 'B']
 
+    # Sort in reverse order of time stamp for 'Done'
+    dnames = [job for job in jobs.keys() if jobs[job]['status'] == 'D']
+    dnames.sort(lambda x, y: jobs[y].get('timestamp', 0) - jobs[x].get('timestamp', 0))
+    done = [xml_name(jobs[job]['title']) for job in dnames]
+
+
     return done, good, bad
 
 def ShowRssLog(feed, all):
@@ -2575,7 +2645,7 @@ LIST_EMAIL = (
     'email_server', 'email_to', 'email_from',
     'email_account', 'email_pwd', 'email_dir', 'email_rss'
 )
-LIST_GROWL = ('growl_enable', 'growl_server', 'growl_password', 'ntfosd_enable')
+LIST_GROWL = ('growl_enable', 'growl_server', 'growl_password', 'ntfosd_enable', 'ncenter_enable')
 
 class ConfigNotify(object):
     def __init__(self, web_dir, root, prim):
@@ -2586,7 +2656,7 @@ class ConfigNotify(object):
 
     @cherrypy.expose
     def index(self, **kwargs):
-        if cfg.configlock():
+        if cfg.configlock() or not check_access():
             return Protected()
 
         conf, pnfo_list, bytespersec = build_header(self.__prim, self.__web_dir)
@@ -2595,11 +2665,15 @@ class ConfigNotify(object):
         conf['lastmail'] = self.__lastmail
         conf['have_growl'] = True
         conf['have_ntfosd'] = sabnzbd.growler.have_ntfosd()
+        conf['have_ncenter'] = sabnzbd.DARWIN_ML and bool(sabnzbd.growler.ncenter_path())
 
         for kw in LIST_EMAIL:
             conf[kw] = config.get_config('misc', kw).get_string()
         for kw in LIST_GROWL:
             conf[kw] = config.get_config('growl', kw).get_string()
+        conf['notify_list'] = NOTIFY_KEYS
+        conf['notify_classes'] = cfg.notify_classes.get_string()
+        conf['notify_texts'] = sabnzbd.growler.NOTIFICATION
 
         template = Template(file=os.path.join(self.__web_dir, 'config_notify.tmpl'),
                             filter=FILTER, searchList=[conf], compilerSettings=DIRECTIVES)
@@ -2618,6 +2692,7 @@ class ConfigNotify(object):
             msg = config.get_config('growl', kw).set(platform_encode(kwargs.get(kw)))
             if msg:
                 return badParameterResponse(T('Incorrect value for %s: %s') % (kw, unicoder(msg)))
+        cfg.notify_classes.set(kwargs.get('notify_classes', ''))
 
         config.save_config()
         self.__lastmail = None
@@ -2668,6 +2743,7 @@ def rss_history(url, limit=50, search=None):
             item.link = history['url_info']
         else:
             item.link = url
+            item.guid = history['nzo_id']
 
         stageLine = []
         for stage in history['stage_log']:
