@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2011 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -28,6 +28,7 @@ import logging
 
 import sabnzbd
 from sabnzbd.constants import *
+import sabnzbd.cfg
 
 try:
     from OpenSSL import SSL
@@ -84,23 +85,32 @@ def request_server_info(server):
 
 
 def GetServerParms(host, port):
-    # Make sure port is numeric (unicode input not supported)
+    """ Return processed getaddrinfo() for server
+    """
     try:
         int(port)
     except:
-        # Could do with a warning here
         port = 119
+    opt = sabnzbd.cfg.ipv6_servers()
     try:
-        # Standard IPV4
-        return socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM)
+        # Standard IPV4 or IPV6
+        ips = socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM)
+        if opt == 2 or (_EXTERNAL_IPV6 and opt == 1):
+            # IPv6 reachable and allowed, or forced by user
+            return ips
+        else:
+            # IPv6 unreachable or not allowed by user
+            return [ip for ip in ips if ':' not in ip[4][0]]
     except:
-        try:
-            # Try IPV6 explicitly
-            return socket.getaddrinfo(host, port, socket.AF_INET6,
-                                      socket.SOCK_STREAM, socket.IPPROTO_IP, socket.AI_CANONNAME)
-        except:
-            # Nothing found!
-            return None
+        if opt == 2 or (_EXTERNAL_IPV6 and opt == 1):
+            try:
+                # Try IPV6 explicitly
+                return socket.getaddrinfo(host, port, socket.AF_INET6,
+                                          socket.SOCK_STREAM, socket.IPPROTO_IP, socket.AI_CANONNAME)
+            except:
+                # Nothing found!
+                pass
+        return None
 
 
 def con(sock, host, port, sslenabled, write_fds, nntp):
@@ -249,7 +259,7 @@ class NewsWrapper(object):
         self.force_login = False
 
     def init_connect(self, write_fds):
-        self.nntp = NNTP(self.server.host, self.server.port, self.server.info, self.server.ssl, self,
+        self.nntp = NNTP(self.server.hostip, self.server.port, self.server.info, self.server.ssl, self,
                          self.server.username, self.server.password, self.blocking, write_fds)
         self.recv = self.nntp.sock.recv
 
@@ -307,9 +317,17 @@ class NewsWrapper(object):
 
         self.timeout = time.time() + self.server.timeout
 
-    def body(self):
+    def body(self, precheck):
         self.timeout = time.time() + self.server.timeout
-        command = 'BODY <%s>\r\n' % (self.article.article)
+        if precheck:
+            if self.server.have_stat:
+                command = 'STAT <%s>\r\n' % (self.article.article)
+            else:
+                command = 'HEAD <%s>\r\n' % (self.article.article)
+        elif self.server.have_body:
+            command = 'BODY <%s>\r\n' % (self.article.article)
+        else:
+            command = 'ARTICLE <%s>\r\n' % (self.article.article)
         self.nntp.sock.sendall(command)
 
     def send_group(self, group):
@@ -337,6 +355,11 @@ class NewsWrapper(object):
 
         self.data += chunk
         new_lines = self.data.split('\r\n')
+        # See if incorrect newline-only was used
+        # Do this as a special case to prevent using extra memory
+        # for normal articles
+        if len(new_lines) == 1 and '\r' not in self.data:
+            new_lines = self.data.split('\n')
 
         self.data = new_lines.pop()
         self.lines.extend(new_lines)
@@ -405,3 +428,25 @@ class SSLConnection(object):
                 return apply(self._ssl_conn.%s, args)
             finally:
                 self._lock.release()\n""" % (f, f)
+
+
+def test_ipv6():
+    """ Check if external IPv6 addresses are reachable """
+    # Use google.com to test IPv6 access
+    try:
+        info = socket.getaddrinfo('www.google.com', 80, socket.AF_INET6, socket.SOCK_STREAM,
+                                  socket.IPPROTO_IP, socket.AI_CANONNAME)
+    except:
+        return False
+
+    try:
+        af, socktype, proto, canonname, sa = info[0]
+        sock = socket.socket(af, socktype, proto)
+        sock.settimeout(6)
+        sock.connect(sa[0:2])
+        sock.close()
+        return True
+    except:
+        return False
+
+_EXTERNAL_IPV6 = test_ipv6()
