@@ -220,7 +220,7 @@ class NzbQueue(TryList):
             if save_nzo is None or nzo is save_nzo:
                 sabnzbd.save_data(nzo, nzo.nzo_id, nzo.workpath)
                 if not nzo.futuretype:
-                    nzo.save_attribs()
+                    nzo.save_to_disk()
 
         sabnzbd.save_admin((QUEUE_VERSION, nzo_ids, []), QUEUE_FILE_NAME)
 
@@ -308,11 +308,11 @@ class NzbQueue(TryList):
                 self.set_priority(nzo_id, prio)
 
     @synchronized(NZBQUEUE_LOCK)
-    def change_name(self, nzo_id, name):
+    def change_name(self, nzo_id, name, password=None):
         if nzo_id in self.__nzo_table:
             nzo = self.__nzo_table[nzo_id]
             if not nzo.futuretype:
-                nzo.set_final_name_pw(name)
+                nzo.set_final_name_pw(name, password)
             else:
                 # Reset url fetch wait time
                 nzo.wait = None
@@ -595,7 +595,7 @@ class NzbQueue(TryList):
                 return nzo_id_pos1
 
             nzo.priority = priority
-            nzo.save_attribs()
+            nzo.save_to_disk()
 
             if nzo_id_pos1 != -1:
                 del self.__nzo_list[nzo_id_pos1]
@@ -755,7 +755,7 @@ class NzbQueue(TryList):
         if not nzo.deleted:
             nzo.deleted = True
             if nzo.precheck:
-                nzo.save_attribs()
+                nzo.save_to_disk()
                 # Check result
                 enough, ratio = nzo.check_quality()
                 if enough:
@@ -886,7 +886,7 @@ def _nzo_date_cmp(nzo1, nzo2):
     return cmp(avg_date1, avg_date2)
 
 def _nzo_name_cmp(nzo1, nzo2):
-    return cmp(nzo1.filename, nzo2.filename)
+    return cmp(nzo1.filename.lower(), nzo2.filename.lower())
 
 def _nzo_size_cmp(nzo1, nzo2):
     return cmp(nzo1.bytes, nzo2.bytes)
