@@ -37,7 +37,7 @@ import sabnzbd
 from sabnzbd.constants import sample_match, GIGI, ATTRIB_FILE, JOB_ADMIN, \
                               DEFAULT_PRIORITY, LOW_PRIORITY, NORMAL_PRIORITY, \
                               HIGH_PRIORITY, PAUSED_PRIORITY, TOP_PRIORITY, DUP_PRIORITY, \
-                              Status
+                              RENAMES_FILE, Status
 from sabnzbd.misc import to_units, cat_to_opts, cat_convert, sanitize_foldername, \
                          get_unique_path, get_admin_path, remove_all, format_source_url, \
                          sanitize_filename, globber, sanitize_foldername, int_conv, \
@@ -55,6 +55,7 @@ SUBJECT_FN_MATCHER = re.compile(r'"([^"]*)"')
 RE_SAMPLE = re.compile(sample_match, re.I)
 PROBABLY_PAR2_RE = re.compile(r'(.*)\.vol(\d*)\+(\d*)\.par2', re.I)
 REJECT_PAR2_RE = re.compile(r'\.par2\.\d+', re.I) # Reject duplicate par2 files
+RE_NORMAL_NAME = re.compile(r'\.\w{2,5}$') # Test reasonably sized extension at the end
 
 ################################################################################
 # Article                                                                      #
@@ -154,6 +155,7 @@ NzbFileMapper = (
     ('import_finished',              'import_finished'),
     ('md5sum',                       'md5sum'),
     ('valid',                        'valid'),
+    ('completed',                    'completed')
 )
 
 
@@ -169,9 +171,7 @@ class NzbFile(TryList):
         self.filename = None
         self.type = None
 
-        match = re.search(SUBJECT_FN_MATCHER, subject)
-        if match:
-            self.filename = match.group(1).strip('"')
+        self.filename = name_extractor(subject)
 
         self.is_par2 = False
         self.vol = None
@@ -191,6 +191,7 @@ class NzbFile(TryList):
         self.nzo = nzo
         self.nzf_id = sabnzbd.get_new_id("nzf", nzo.workpath)
         self.deleted = False
+        self.completed = False
 
         self.valid = False
         self.import_finished = False
@@ -321,6 +322,11 @@ class NzbParser(xml.sax.handler.ContentHandler):
         self.in_group = False
         self.in_segments = False
         self.in_segment = False
+        self.in_head = False
+        self.in_meta = False
+        self.meta_type = ''
+        self.meta_types = {}
+        self.meta_content = []
         self.filename = ''
         self.avg_age = 0
         self.valids = 0
@@ -348,12 +354,8 @@ class NzbParser(xml.sax.handler.ContentHandler):
             self.in_segments = True
 
         elif name == 'file' and self.in_nzb:
-            subject = attrs.get('subject', '')
-            match = re.search(SUBJECT_FN_MATCHER, subject)
-            if match:
-                self.filename = match.group(1).strip('"').strip()
-            else:
-                self.filename = subject.strip()
+            subject = attrs.get('subject', '').strip()
+            self.filename = subject
 
             if self.filter and RE_SAMPLE.search(subject):
                 logging.info('Skipping sample file %s', subject)
@@ -377,6 +379,16 @@ class NzbParser(xml.sax.handler.ContentHandler):
         elif name == 'groups' and self.in_nzb and self.in_file:
             self.in_groups = True
 
+        elif name == 'head' and self.in_nzb:
+            self.in_head = True
+
+        elif name == 'meta' and self.in_nzb and self.in_head:
+            self.in_meta = True
+            meta_type = attrs.get('type')
+            if meta_type:
+                self.meta_type = meta_type.lower()
+            self.meta_content = []
+
         elif name == 'nzb':
             self.in_nzb = True
 
@@ -385,6 +397,8 @@ class NzbParser(xml.sax.handler.ContentHandler):
             self.group_name.append(content)
         elif self.in_segment:
             self.article_id.append(content)
+        elif self.in_meta:
+            self.meta_content.append(content)
 
     def endElement(self, name):
         if name == 'group' and self.in_group:
@@ -439,12 +453,24 @@ class NzbParser(xml.sax.handler.ContentHandler):
                     sabnzbd.remove_data(nzf.nzf_id, self.nzo.workpath)
                 self.skipped_files += 1
 
+        elif name == 'head':
+            self.in_head = False
+
+        elif name == 'meta':
+            self.in_meta = False
+            if self.meta_type:
+                if self.meta_type not in self.meta_types:
+                    self.meta_types[self.meta_type] = []
+                self.meta_types[self.meta_type].append(''.join(self.meta_content))
+
         elif name == 'nzb':
             self.in_nzb = False
 
     def endDocument(self):
         """ End of the file """
         self.nzo.groups = self.groups
+        self.nzo.meta = self.meta_types
+        logging.debug('META-DATA = %s', self.nzo.meta)
         files = max(1, self.valids)
         self.nzo.avg_stamp = self.avg_age / files
         self.nzo.avg_date = datetime.datetime.fromtimestamp(self.avg_age / files)
@@ -504,7 +530,9 @@ NzbObjectMapper = (
     ('oversized',                    'oversized'),     # Was detected as oversized
     ('create_group_folder',          'create_group_folder'),
     ('precheck',                     'precheck'),
-    ('incomplete',                   'incomplete')     # Was detected as incomplete
+    ('incomplete',                   'incomplete'),    # Was detected as incomplete
+    ('reuse',                        'reuse'),
+    ('meta',                         'meta')           # Meta-date from 1.1 type NZB
 )
 
 class NzbObject(TryList):
@@ -540,6 +568,7 @@ class NzbObject(TryList):
         self.work_name = work_name
         self.final_name = work_name
 
+        self.meta = {}
         self.created = False        # dirprefixes + work_name created
         self.bytes = 0              # Original bytesize
         self.bytes_downloaded = 0   # Downloaded byte
@@ -591,6 +620,7 @@ class NzbObject(TryList):
         self.oversized = False
         self.precheck = False
         self.incomplete = False
+        self.reuse = reuse
         if self.status == Status.QUEUED and not reuse:
             self.precheck = cfg.pre_check()
             if self.precheck:
@@ -698,12 +728,23 @@ class NzbObject(TryList):
 
         if not self.files and not reuse:
             self.purge_data(keep_basic=False)
-            if self.url:
-                logging.warning(Ta('Empty NZB file %s') + ' [%s]', filename, self.url)
+            if cfg.warn_empty_nzb():
+                mylog = logging.warning
             else:
-                logging.warning(Ta('Empty NZB file %s'), filename)
+                mylog = logging.info
+            if self.url:
+                mylog(Ta('Empty NZB file %s') + ' [%s]', filename, self.url)
+            else:
+                mylog(Ta('Empty NZB file %s'), filename)
             raise ValueError
 
+        if cat is None:
+            for metacat in self.meta.get('category', ()):
+                metacat = cat_convert(metacat)
+                if metacat:
+                    cat = metacat
+                    break
+
         if cat is None:
             for grp in self.groups:
                 cat = cat_convert(grp)
@@ -830,11 +871,11 @@ class NzbObject(TryList):
             # Move only when not current NZF and filename was extractable from subject
             if name and nzf is not xnzf:
                 head, vol, block = analyse_par2(name)
-                # When only subject is known, it's enough that that 'parset' is in subject
-                if head and lparset in head.lower():
+                if head and matcher(lparset, head.lower()):
                     xnzf.set_par2(parset, vol, block)
                     self.extrapars[parset].append(xnzf)
-                    self.files.remove(xnzf)
+                    if not self.precheck:
+                        self.files.remove(xnzf)
 
     def handle_par2(self, nzf, file_done):
         """ Check if file is a par2 and build up par2 collection
@@ -848,6 +889,9 @@ class NzbObject(TryList):
                 head, vol, block = analyse_par2(fn)
                 ## Is a par2file and repair mode activated
                 if head and (self.repair or cfg.allow_streaming()):
+                    ## Skip if mini-par2 is not complete and there are more par2 files
+                    if not block and nzf.bytes_left and self.extrapars.get(head):
+                        return
                     nzf.set_par2(head, vol, block)
                     ## Already got a parfile for this set?
                     if head in self.partable:
@@ -875,7 +919,7 @@ class NzbObject(TryList):
                     ## initialparfile
                     else:
                         self.postpone_pars(nzf, head)
-                ## Is not a par2file or nothing todo
+                ## Is not a par2file or nothing to do
                 else:
                     pass
         ## No filename in seg 1? Probably not uu or yenc encoded
@@ -890,6 +934,14 @@ class NzbObject(TryList):
 
         if file_done:
             self.remove_nzf(nzf)
+            if not self.reuse and cfg.fail_hopeless() and not self.check_quality(99)[0]:
+                #set the nzo status to return "Queued"
+                self.status = Status.QUEUED
+                self.set_download_report()
+                self.fail_msg = T('Aborted, cannot be completed')
+                self.set_unpack_info('Download', self.fail_msg, unique=False)
+                logging.debug('Abort job "%s", due to impossibility to complete it', self.final_name_pw_clean)
+                return True, True, True
 
         if reset:
             self.reset_try_list()
@@ -912,6 +964,15 @@ class NzbObject(TryList):
         """
         # Get a list of already present files
         files = [os.path.basename(f) for f in globber(wdir) if os.path.isfile(f)]
+
+        # Substitute renamed files
+        renames = sabnzbd.load_data(RENAMES_FILE, self.workpath, remove=True)
+        if renames:
+            for name in renames:
+                if name in files:
+                    files.remove(name)
+                    files.append(renames[name])
+
         # Looking for the longest name first, minimizes the chance on a mismatch
         files.sort(lambda x, y: len(y) - len(x))
 
@@ -926,6 +987,7 @@ class NzbObject(TryList):
                 if (nzf.filename == filename) or (subject == filename) or (filename in subject):
                     nzf.filename = filename
                     nzf.completed = True
+                    nzf.bytes_left = 0
                     self.handle_par2(nzf, file_done=True)
                     self.remove_nzf(nzf)
                     nzfs.remove(nzf)
@@ -942,6 +1004,8 @@ class NzbObject(TryList):
                 self.files_table[nzf.nzf_id] = nzf
                 self.bytes += nzf.bytes
                 nzf.filename = filename
+                nzf.completed = True
+                nzf.bytes_left = 0
                 self.handle_par2(nzf, file_done=True)
                 self.remove_nzf(nzf)
                 logging.info('File %s added to job', filename)
@@ -958,6 +1022,7 @@ class NzbObject(TryList):
 
     def set_pp(self, value):
         self.repair, self.unpack, self.delete = sabnzbd.pp_to_opts(value)
+        self.save_to_disk()
 
     @property
     def final_name_pw(self):
@@ -986,11 +1051,15 @@ class NzbObject(TryList):
         else:
             return self.final_name
 
-    def set_final_name_pw(self, name):
+    def set_final_name_pw(self, name, password=None):
         if isinstance(name, str):
-            name, self.password = scan_password(platform_encode(name))
+            if password:
+                name = platform_encode(name)
+                self.password = platform_encode(password)
+            else:
+                name, self.password = scan_password(platform_encode(name))
             self.final_name = sanitize_foldername(name)
-            self.save_attribs()
+            self.save_to_disk()
 
     def pause(self):
         self.status = 'Paused'
@@ -1018,7 +1087,7 @@ class NzbObject(TryList):
         self.partable.pop(setname)
 
     __re_quick_par2_check = re.compile('\.par2\W*', re.I)
-    def check_quality(self):
+    def check_quality(self, req_ratio=0):
         """ Determine amount of articles present on servers
             and return (gross available, nett) bytes
         """
@@ -1029,7 +1098,8 @@ class NzbObject(TryList):
         for nzf_id in self.files_table:
             nzf = self.files_table[nzf_id]
             assert isinstance(nzf, NzbFile)
-            short += nzf.bytes_left
+            if nzf.deleted:
+                short += nzf.bytes_left
             if self.__re_quick_par2_check.search(nzf.subject):
                 pars += nzf.bytes
                 anypars = True
@@ -1038,7 +1108,7 @@ class NzbObject(TryList):
         have = need + pars - short
         ratio = float(have) / float(max(1, need))
         if anypars:
-            enough = ratio * 100.0 >= float(cfg.req_completion_rate())
+            enough = ratio * 100.0 >= (req_ratio or float(cfg.req_completion_rate()))
         else:
             enough = have >= need
         logging.debug('Download Quality: enough=%s, have=%s, need=%s, ratio=%s', enough, have, need, ratio)
@@ -1062,15 +1132,18 @@ class NzbObject(TryList):
             msg1 = T('Downloaded in %s at an average of %sB/s') % (complete_time, to_units(avg_bps*1024, dec_limit=1))
             bad = self.nzo_info.get('bad_art_log', [])
             miss = self.nzo_info.get('missing_art_log', [])
+            killed = self.nzo_info.get('killed_art_log', [])
             dups = self.nzo_info.get('dup_art_log', [])
-            msg2 = msg3 = msg4 = ''
+            msg2 = msg3 = msg4 = msg5 = ''
             if bad:
                 msg2 = ('<br/>' + T('%s articles were malformed')) % len(bad)
             if miss:
                 msg3 = ('<br/>' + T('%s articles were missing')) % len(miss)
             if dups:
                 msg4 = ('<br/>' + T('%s articles had non-matching duplicates')) % len(dups)
-            msg = ''.join((msg1, msg2, msg3, msg4,))
+            if killed:
+                msg5 = ('<br/>' + T('%s articles were removed')) % len(killed)
+            msg = ''.join((msg1, msg2, msg3, msg4, msg5, ))
             self.set_unpack_info('Download', msg, unique=True)
             if self.url:
                 self.set_unpack_info('Source', format_source_url(self.url), unique=True)
@@ -1242,6 +1315,15 @@ class NzbObject(TryList):
             bytes_left += nzf.bytes_left
         return bytes_left
 
+    def total_and_remaining(self):
+        """ Return total and remaining bytes """
+        bytes = 0
+        bytes_left = 0
+        for nzf in self.files:
+            bytes += nzf.bytes
+            bytes_left += nzf.bytes_left
+        return bytes, bytes_left
+
     def gather_info(self, for_cli = False):
         bytes_left_all = 0
 
@@ -1335,6 +1417,12 @@ class NzbObject(TryList):
     def repair_opts(self):
         return self.repair, self.unpack, self.delete
 
+    def save_to_disk(self):
+        """ Save job's admin to disk """
+        self.save_attribs()
+        if self.nzo_id:
+            sabnzbd.save_data(self, self.nzo_id, self.workpath)
+            
     def save_attribs(self):
         set_attrib_file(self.workpath, (self.cat, self.pp, self.script, self.priority, self.final_name_pw_clean, self.url))
 
@@ -1374,6 +1462,8 @@ class NzbObject(TryList):
         self.pp_active = False
         self.avg_stamp = time.mktime(self.avg_date.timetuple())
         self.wait = None
+        if self.meta is None:
+            self.meta = {}
         TryList.__init__(self)
 
 
@@ -1384,8 +1474,9 @@ class NzbObject(TryList):
 #-------------------------------------------------------------------------------
 
 def nzf_get_filename(nzf):
-    # Return filename, if the filename not set, try the
-    # the full subject line instead. Can produce non-ideal results
+    """ Return filename, if the filename not set, try the
+        the full subject line instead. Can produce non-ideal results
+    """
     name = nzf.filename
     if not name:
         name = nzf.subject
@@ -1394,8 +1485,31 @@ def nzf_get_filename(nzf):
     return name.lower()
 
 
+def get_ext_list():
+    """ Return priority extenstion list, with extensions starting with a period
+    """
+    exts = []
+    for ext in cfg.prio_sort_list():
+        ext = ext.strip()
+        if not ext.startswith('.'):
+            ext = '.' + ext
+        exts.append(ext)
+    return exts
+
+
+def ext_on_list(name, lst):
+    """ Return True if `name` contains any extension in `lst`
+    """
+    for ext in lst:
+        if name.rfind(ext) >= 0:
+            return True
+    return False
+
+
 def nzf_cmp_date(nzf1, nzf2):
-    # Compare files based on date, but give vol-par files preference
+    """ Compare files based on date, but give vol-par files preference.
+        Wrapper needed, because `cmp` function doesn't handle extra parms.
+    """
     return nzf_cmp_name(nzf1, nzf2, name=False)
 
 
@@ -1423,6 +1537,16 @@ def nzf_cmp_name(nzf1, nzf2, name=True):
     if is_par2 and not is_par1:
         return -1
 
+    # Anything with a priority extention goes first
+    ext_list = get_ext_list()
+    if ext_list:
+        onlist1 = ext_on_list(name1, ext_list)
+        onlist2 = ext_on_list(name2, ext_list)
+        if onlist1 and not onlist2:
+            return -1
+        if onlist2 and not onlist1:
+            return 1
+
     if name:
         # Prioritise .rar files above any other type of file (other than vol-par)
         # Useful for nzb streaming
@@ -1521,8 +1645,10 @@ def get_attrib_file(path, size):
         return [None for n in xrange(size)]
 
     for n in xrange(size):
-        line = f.readline().strip('\n ')
+        line = f.readline().strip('\r\n ')
         if line:
+            if line.lower() == 'none':
+                line = None
             try:
                 line = int(line)
             except:
@@ -1561,7 +1687,28 @@ def analyse_par2(name):
             vol = m.group(2)
             block = m.group(3)
         elif name.lower().find('.par2') > 0:
-            head = os.path.splitext(name)[0]
+            head = os.path.splitext(name)[0].strip()
         else:
             head = None
     return head, vol, block
+
+
+def name_extractor(subject):
+    """ Try to extract a file name from a subject line, return `subject` if in doubt
+    """
+    result = subject
+    for name in re.findall(SUBJECT_FN_MATCHER, subject):
+        name = name.strip(' "')
+        if name and RE_NORMAL_NAME.search(name):
+            result = name
+    return platform_encode(result)
+
+
+def matcher(pattern, txt):
+    """ Return True if `pattern` is sufficiently equal to `txt`
+    """
+    if txt.endswith(pattern):
+        txt = txt[:txt.rfind(pattern)].strip()
+        return (not txt) or txt.endswith('"')
+    else:
+        return False
