@@ -31,7 +31,7 @@ from sabnzbd.constants import *
 from sabnzbd.decorators import synchronized
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
-from sabnzbd.misc import cat_convert, sanitize_foldername, wildcard_to_re, cat_to_opts
+from sabnzbd.misc import cat_convert, sanitize_foldername, wildcard_to_re, cat_to_opts, match_str
 import sabnzbd.emailer as emailer
 from sabnzbd.encoding import latin1, unicoder, xml_name
 
@@ -84,6 +84,16 @@ def run_method():
     else:
         return None
 
+def next_run(t=None):
+    global __RSS
+    if __RSS:
+        if t:
+            __RSS.next_run = t
+        else:
+            return __RSS.next_run
+    else:
+        return time.time()
+
 def save():
     global __RSS
     if __RSS: __RSS.save()
@@ -159,6 +169,8 @@ class RSSQueue(object):
                 return False
 
         self.jobs = {}
+        self.next_run = time.time()
+
         try:
             defined = config.get_rss().keys()
             feeds = sabnzbd.load_admin(RSS_FILE_NAME)
@@ -303,13 +315,7 @@ class RSSQueue(object):
         regcount = len(regexes)
 
         # Set first if this is the very first scan of this URI
-        if feed not in self.jobs:
-            self.jobs[feed] = {}
-        first = not bool(self.jobs[feed])
-
-        jobs = self.jobs[feed]
-
-        first = first and ignoreFirst
+        first = (feed not in self.jobs) and ignoreFirst
 
         # Add sabnzbd's custom User Agent
         feedparser.USER_AGENT = 'SABnzbd+/%s' % sabnzbd.version.__version__
@@ -319,6 +325,8 @@ class RSSQueue(object):
             uri += '&dl=1'
 
         # Read the RSS feed
+        msg = None
+        entries = None
         if readout:
             uri = uri.replace(' ', '%20')
             logging.debug("Running feedparser on %s", uri)
@@ -343,9 +351,18 @@ class RSSQueue(object):
             if not entries:
                 msg = Ta('RSS Feed %s was empty') % uri
                 logging.info(msg)
+
+        if feed not in self.jobs:
+            self.jobs[feed] = {}
+        jobs = self.jobs[feed]
+        if readout:
+            if not entries:
                 return unicoder(msg)
         else:
             entries = jobs.keys()
+            # Sort in the order the jobs came from the feed
+            entries.sort(lambda x, y: jobs[x].get('order', 0) - jobs[y].get('order', 0))
+
 
         order = 0
         # Filter out valid new links
@@ -483,12 +500,13 @@ class RSSQueue(object):
         """
         if not sabnzbd.PAUSED_ALL:
             active = False
+            if self.next_run < time.time():
+                self.next_run = time.time() + cfg.rss_rate.get() * 60
             feeds = config.get_rss()
             for feed in feeds.keys():
                 try:
                     if feeds[feed].enable.get():
-                        if not active:
-                            logging.info('Starting scheduled RSS read-out')
+                        logging.info('Starting scheduled RSS read-out for "%s"', feed)
                         active = True
                         self.run_feed(feed, download=True, ignoreFirst=True)
                         # Wait 15 seconds, else sites may get irritated
@@ -502,7 +520,7 @@ class RSSQueue(object):
                     pass
             if active:
                 self.save()
-                logging.info('Finished scheduled RSS read-out')
+                logging.info('Finished scheduled RSS read-outs')
 
 
     @synchronized(LOCK)
@@ -556,7 +574,7 @@ class RSSQueue(object):
                     self.jobs[feed][item]['status'] = 'D-'
 
 
-RE_NEWZBIN = re.compile(r'(newz)(bin|xxx|bin2)\.[\w]+/browse/post/(\d+)', re.I)
+RE_NEWZBIN = re.compile(r'(newz)(bin|xxx|bin2|xxx2)\.[\w]+/browse/post/(\d+)', re.I)
 
 def _HandleLink(jobs, link, title, flag, orgcat, cat, pp, script, download, star, order,
                 priority=NORMAL_PRIORITY, rule=0):
@@ -617,7 +635,7 @@ def _get_link(uri, entry):
     link = None
     category = ''
     uri = uri.lower()
-    if 'newzbin.' in uri or 'newzxxx.'in uri or 'newzbin2.' in uri:
+    if 'newzbin.' in uri or 'newzxxx.' in uri or 'newzbin2.' in uri or 'newzxxx2.' in uri:
         link = entry.link
         if not (link and '/post/' in link.lower()):
             # Use alternative link
@@ -656,12 +674,12 @@ def _get_link(uri, entry):
 def special_rss_site(url):
     """ Return True if url describes an RSS site with odd titles
     """
-    return cfg.rss_filenames() or 'nzbindex.nl/' in url or 'nzbindex.com/' in url or 'nzbclub.com/' in url
+    return cfg.rss_filenames() or match_str(url, cfg.rss_odd_titles())
 
 
 _ENCL_SITES = ('nzbindex.nl', 'nzbindex.com', 'animeusenet.org', 'nzbclub.com')
 def encl_sites(url, link):
-    """ Return True if url or link match sites that use enclosures
+    """ Return True if url or link matches sites that use enclosures
     """
     for site in _ENCL_SITES:
         if site in url or (link and site in link):
