@@ -40,7 +40,7 @@ __SCHED = None  # Global pointer to Scheduler instance
 
 RSSTASK_MINUTE = random.randint(0, 59)
 SCHEDULE_GUARD_FLAG = False
-
+PP_PAUSE_EVENT = False
 
 def schedule_guard():
     """ Set flag for scheduler restart """
@@ -53,6 +53,8 @@ def pp_pause():
 def pp_resume():
     PostProcessor.do.paused = False
 
+def pp_pause_event():
+    return PP_PAUSE_EVENT
 
 def init():
     """ Create the scheduler and set all required events
@@ -116,6 +118,8 @@ def init():
         elif action_name == 'rss_scan':
             action = rss.run_method
             rss_planned = True
+        elif action_name == 'remove_failed':
+            action = sabnzbd.api.history_remove_failed
         else:
             logging.warning(Ta('Unknown action: %s'), action_name)
             continue
@@ -134,6 +138,7 @@ def init():
         interval = cfg.rss_rate()
         delay = random.randint(0, interval-1)
         logging.debug("Scheduling RSS interval task every %s min (delay=%s)", interval, delay)
+        sabnzbd.rss.next_run(time.time() + delay * 60)
         __SCHED.add_interval_task(rss.run_method, "RSS", delay*60, interval*60,
                                       kronos.method.sequential, None, None)
         __SCHED.add_single_task(rss.run_method, 'RSS', 15, kronos.method.sequential, None, None)
@@ -149,7 +154,7 @@ def init():
                                  kronos.method.sequential, [], None)
 
 
-    if cfg.newzbin_bookmarks():
+    if False: #cfg.newzbin_bookmarks():
         interval = cfg.bookmark_rate()
         delay = random.randint(0, interval-1)
         logging.debug("Scheduling Bookmark interval task every %s min (delay=%s)", interval, delay)
@@ -164,6 +169,10 @@ def init():
         __SCHED.add_daytime_task(action, 'quota_reset', range(1, 8), None, (hour, minute),
                                  kronos.method.sequential, [], None)
 
+    logging.info('Setting schedule for midnight BPS reset')
+    __SCHED.add_daytime_task(sabnzbd.bpsmeter.midnight_action, 'midnight_bps', range(1, 8), None, (0, 0),
+                             kronos.method.sequential, [], None)
+
 
     # Subscribe to special schedule changes
     cfg.newzbin_bookmarks.callback(schedule_guard)
@@ -221,56 +230,62 @@ def abort():
         __SCHED.running = False
 
 
-def sort_schedules(forward):
+def sort_schedules(all_events, now=None):
     """ Sort the schedules, based on order of happening from now
-        forward: assume expired daily event to occur tomorrow
+        `all_events=True`: Return an event for each active day
+        `all_events=False`: Return only first occurring event of the week
+        `now` : for testing: simulated localtime()
     """
 
+    day_min = 24 * 60
+    week_min = 7 * day_min
     events = []
-    now = time.localtime()
-    now_hm = int(now[3])*60 + int(now[4])
-    now = int(now[6])*24*60 + now_hm
+
+    now = now or time.localtime()
+    now_hm = now[3] * 60 + now[4]
+    now = now[6] * day_min + now_hm
 
     for schedule in cfg.schedules():
         parms = None
         try:
-            m, h, d, action, parms = schedule.split(None, 4)
+            m, h, dd, action, parms = schedule.split(None, 4)
         except:
             try:
-                m, h, d, action = schedule.split(None, 3)
+                m, h, dd, action = schedule.split(None, 3)
             except:
                 continue # Bad schedule, ignore
         action = action.strip()
-        try:
-            then = int(h)*60 + int(m)
-            if d == '*':
-                d = int(now/(24*60))
-                if forward and (then < now_hm): d = (d + 1) % 7
-            else:
-                d = int(d)-1
-            then = d*24*60 + then
-        except:
+        if dd == '*':
+            dd = '1234567'
+        if not dd.isdigit():
             continue # Bad schedule, ignore
+        for d in dd:
+            then = (int(d) - 1) * day_min + int(h) * 60 + int(m)
+            dif = then - now
+            if all_events and dif < 0:
+                # Expired event will occur again after a week
+                dif = dif + week_min
 
-        dif = then - now
-        if dif < 0: dif = dif + 7*24*60
+            events.append((dif, action, parms, schedule))
+            if not all_events:
+                break
 
-        events.append((dif, action, parms, schedule))
-
-    events.sort(lambda x, y: x[0]-y[0])
+    events.sort(lambda x, y: x[0] - y[0])
     return events
 
 
 def analyse(was_paused=False):
     """ Determine what pause/resume state we would have now.
     """
+    global PP_PAUSE_EVENT
+    PP_PAUSE_EVENT = False
     paused = None
     paused_all = False
     pause_post = False
     speedlimit = None
     servers = {}
 
-    for ev in sort_schedules(forward=False):
+    for ev in sort_schedules(all_events=True):
         logging.debug('Schedule check result = %s', ev)
         action = ev[1]
         try:
@@ -281,13 +296,16 @@ def analyse(was_paused=False):
             paused = True
         elif action == 'pause_all':
             paused_all = True
+            PP_PAUSE_EVENT = True
         elif action == 'resume':
             paused = False
             paused_all = False
         elif action == 'pause_post':
             pause_post = True
+            PP_PAUSE_EVENT = True
         elif action == 'resume_post':
             pause_post = False
+            PP_PAUSE_EVENT = True
         elif action == 'speedlimit' and value!=None:
             speedlimit = int(ev[2])
         elif action == 'enable_server':
