@@ -62,6 +62,7 @@ SKIN_TEXT = {
     'sch-resume_post'    : TT('Resume post-processing'), #:  #: Config->Scheduler
     'sch-scan_folder'    : TT('Scan watched folder'), #:  #: Config->Scheduler
     'sch-rss_scan'       : TT('Read RSS feeds'), #:  #: Config->Scheduler
+    'sch-remove_failed'  : TT('Remove failed jobs'), #: Config->Scheduler
 
 # General texts
     'default' : TT('Default'), #: Default value, used in dropdown menus
@@ -298,6 +299,8 @@ SKIN_TEXT = {
     'explain-https_cert' : TT('File name or path to HTTPS Certificate.'),
     'opt-https_key' : TT('HTTPS Key'),
     'explain-https_key' : TT('File name or path to HTTPS Key.'),
+    'opt-https_chain' : TT('HTTPS Chain Certifcates'),
+    'explain-https_chain' : TT('File name or path to HTTPS Chain.'),
     'tuning' : TT('Tuning'),
     'opt-refresh_rate' : TT('Queue auto refresh interval:'),
     'explain-refresh_rate' : TT('Refresh interval of the queue web-interface page(sec, 0= none).'),
@@ -377,13 +380,14 @@ SKIN_TEXT = {
     'explain-top_only' : TT('Enable for less memory usage. Disable to prevent slow jobs from blocking the queue.'),
     'opt-safe_postproc' : TT('Post-Process Only Verified Jobs'),
     'explain-safe_postproc' : TT('Only perform post-processing on jobs that passed all PAR2 checks.'),
-    'opt-pause_on_pwrar' : TT('Pause job when encrypted RAR is downloaded'),
-    'explain-pause_on_pwrar' : TT('You\'ll need to set a password and resume the job.'),
+    'opt-pause_on_pwrar' : TT('Action when encrypted RAR is downloaded'),
+    'explain-pause_on_pwrar' : TT('In case of "Pause", you\'ll need to set a password and resume the job.'),
     'opt-no_dupes' : TT('Detect Duplicate Downloads'),
     'explain-no_dupes' : TT('Detect identically named NZB files (requires NZB backup option) and duplicate titles across RSS feeds.'),
     'nodupes-off' : TT('Off'), #: Three way switch for duplicates
     'nodupes-ignore' : TT('Discard'), #: Three way switch for duplicates
     'nodupes-pause' : TT('Pause'), #: Three way switch for duplicates
+    'abort' : TT('Abort'), #: Three way switch for encrypted posts
     'opt-sfv_check' : TT('Enable SFV-based checks'),
     'explain-sfv_check' : TT('Do an extra verification based on SFV files.'),
     'opt-unpack_check' : TT('Check result of unpacking'),
@@ -455,6 +459,8 @@ SKIN_TEXT = {
     'explain-max_art_tries' : TT('Maximum number of retries per server'),
     'opt-max_art_opt' : TT('Only for optional servers'),
     'explain-max_art_opt' : TT('Apply maximum retries only to optional servers'),
+    'opt-fail_hopeless' : TT('Abort jobs that cannot be completed'),
+    'explain-fail_hopeless' : TT('When during download it becomes clear that too much data is missing, abort the job'),
 
 
 # Config->Server
@@ -551,6 +557,10 @@ SKIN_TEXT = {
     'explain-growl_password' : TT('Optional password for Growl server'), #: Don't translate "Growl"
     'opt-ntfosd_enable' : TT('Enable NotifyOSD'), #: Don't translate "NotifyOSD"
     'explain-ntfosd_enable' : TT('Send notifications to NotifyOSD'), #: Don't translate "NotifyOSD"
+    'opt-ncenter_enable' : TT('Notification Center'),
+    'explain-ncenter_enable' : TT('Send notifications to Notification Center'),
+    'opt-notify_classes' : TT('Notification classes'),
+    'explain-notify_classes' : TT('Enable classes of messages to be reported (none, one or multiple)'),
     'testNotify' : TT('Test Notification'),
 
 # Config->Newzbin
