@@ -61,11 +61,12 @@ COUNTRY_REP = ('(US)', '(UK)', '(EU)', '(CA)', '(YU)', '(VE)', '(TR)', '(CH)', \
                '(AW)', '(AR)', '(AL)', '(AF)')
 
 _RE_ENDEXT = re.compile(r'\.%ext[{}]*$', re.I)
+_RE_ENDFN = re.compile(r'%fn[{}]*$', re.I)
 
-def endswith_ext(path):
-    """ Return True when path ends with '.%ext'
+def ends_in_file(path):
+    """ Return True when path ends with '.%ext' or '%fn'
     """
-    return _RE_ENDEXT.search(path) is not None
+    return bool(_RE_ENDEXT.search(path) or _RE_ENDFN.search(path))
 
 
 def move_to_parent_folder(workdir):
@@ -84,8 +85,8 @@ def move_to_parent_folder(workdir):
         for _file in files:
             path = os.path.join(root, _file)
             new_path = path.replace(workdir, dest)
-            new_path = get_unique_filename(new_path)
-            if not move_to_path(path, new_path, False):
+            ok, new_path = move_to_path(path, new_path)
+            if not ok:
                 return dest, False
 
     cleanup_empty_directories(workdir)
@@ -333,7 +334,7 @@ class SeriesSorter(object):
         sorter = self.sort_string.replace('\\', '/')
         mapping = []
 
-        if endswith_ext(sorter):
+        if ends_in_file(sorter):
             extension = True
             sorter = sorter.replace('.%ext', '')
         else:
@@ -402,9 +403,9 @@ class SeriesSorter(object):
 
         def to_filepath(f, current_path):
             if is_full_path(f):
-                filepath = f.replace('_UNPACK_', '')
+                filepath = os.path.normpath(f)
             else:
-                filepath = os.path.join(current_path, f)
+                filepath = os.path.normpath(os.path.join(current_path, f))
             return filepath
 
         # Create a generator of filepaths, ignore sample files and excluded files (vobs ect)
@@ -425,11 +426,10 @@ class SeriesSorter(object):
         file, filepath, size = largest
         # >20MB
         if filepath and size > 20971520:
-            tmp, self.ext = os.path.splitext(file)
-            self.fname = tmp
+            self.fname, self.ext = os.path.splitext(os.path.split(file)[1])
             newname = "%s%s" % (self.filename_set, self.ext)
             # Replace %fn with the original filename
-            newname = newname.replace('%fn', tmp)
+            newname = newname.replace('%fn', self.fname)
             newpath = os.path.join(current_path, newname)
             # Replace %ext with extension
             newpath = newpath.replace('%ext', self.ext)
@@ -440,7 +440,7 @@ class SeriesSorter(object):
                 except:
                     logging.error("Failed to rename: %s to %s", current_path, newpath)
                     logging.info("Traceback: ", exc_info = True)
-                rename_similar(current_path, self.ext, self.filename_set)
+                rename_similar(current_path, self.ext, self.filename_set, ())
             else:
                 logging.debug('Current path already exists, skipping rename, %s', newpath)
         else:
@@ -594,7 +594,7 @@ class GenericSorter(object):
         sorter = self.sort_string.replace('\\', '/')
         mapping = []
 
-        if endswith_ext(sorter):
+        if ends_in_file(sorter):
             extension = True
             sorter = sorter.replace(".%ext", '')
         else:
@@ -625,6 +625,9 @@ class GenericSorter(object):
         mapping.append(('%decade', self.movie_info['decade']))
         mapping.append(('%0decade', self.movie_info['decade_two']))
 
+        # Original dir name
+        mapping.append(('%dn', self.original_dirname))
+
         path = path_subst(sorter, mapping)
 
         for key, name in REPLACE_AFTER.iteritems():
@@ -654,9 +657,9 @@ class GenericSorter(object):
         logging.debug("Renaming Generic file")
         def filter_files(_file, current_path):
             if is_full_path(_file):
-                filepath = _file.replace('_UNPACK_', '')
+                filepath = os.path.normpath(_file)
             else:
-                filepath = os.path.join(current_path, _file)
+                filepath = os.path.normpath(os.path.join(current_path, _file))
             if os.path.exists(filepath):
                 size = os.stat(filepath).st_size
                 if size >= cfg.movie_rename_limit.get_int() and not RE_SAMPLE.search(_file) \
@@ -672,14 +675,13 @@ class GenericSorter(object):
         if length == 1:
             file = files[0]
             if is_full_path(file):
-                filepath = file.replace('_UNPACK_', '')
+                filepath = os.path.normpath(file)
             else:
-                filepath = os.path.join(current_path, file)
+                filepath = os.path.normpath(os.path.join(current_path, file))
             if os.path.exists(filepath):
-                tmp, ext = os.path.splitext(file)
-                self.fname = tmp
+                self.fname, ext = os.path.splitext(os.path.split(file)[1])
                 newname = "%s%s" % (self.filename_set, ext)
-                newname = newname.replace('%fn', tmp)
+                newname = newname.replace('%fn', self.fname)
                 newpath = os.path.join(current_path, newname)
                 try:
                     logging.debug("Rename: %s to %s", filepath, newpath)
@@ -687,7 +689,7 @@ class GenericSorter(object):
                 except:
                     logging.error(Ta('Failed to rename: %s to %s'), filepath, newpath)
                     logging.info("Traceback: ", exc_info = True)
-                rename_similar(current_path, ext, self.filename_set)
+                rename_similar(current_path, ext, self.filename_set, ())
 
         ## Sequence File Handling
         # if there is more than one extracted file check for CD1/1/A in the title
@@ -696,12 +698,13 @@ class GenericSorter(object):
             # rename files marked as in a set
             if matched_files:
                 logging.debug("Renaming a series of generic files (%s)", matched_files)
+                renamed = matched_files.values()
                 for index, file in matched_files.iteritems():
                     filepath = os.path.join(current_path, file)
-                    tmp, ext = os.path.splitext(file)
-                    self.fname = tmp
+                    renamed.append(filepath)
+                    self.fname, ext = os.path.splitext(os.path.split(file)[1])
                     name = '%s%s' % (self.filename_set, self.extra)
-                    name = name.replace('%1', str(index)).replace('%fn', tmp)
+                    name = name.replace('%1', str(index)).replace('%fn', self.fname)
                     name = name + ext
                     newpath = os.path.join(current_path, name)
                     try:
@@ -710,7 +713,7 @@ class GenericSorter(object):
                     except:
                         logging.error(Ta('Failed to rename: %s to %s'), filepath, newpath)
                         logging.info("Traceback: ", exc_info = True)
-                    rename_similar(current_path, ext, self.filename_set)
+                rename_similar(current_path, ext, self.filename_set, renamed)
             else:
                 logging.debug("Movie files not in sequence %s", _files)
 
@@ -802,7 +805,7 @@ class DateSorter(object):
         sorter = self.sort_string.replace('\\', '/')
         mapping = []
 
-        if endswith_ext(sorter):
+        if ends_in_file(sorter):
             extension = True
             sorter = sorter.replace(".%ext", '')
         else:
@@ -878,18 +881,17 @@ class DateSorter(object):
         #find the master file to rename
         for file in files:
             if is_full_path(file):
-                filepath = file.replace('_UNPACK_', '')
+                filepath = os.path.normpath(file)
             else:
-                filepath = os.path.join(current_path, file)
+                filepath = os.path.normpath(os.path.join(current_path, file))
 
             if os.path.exists(filepath):
                 size = os.stat(filepath).st_size
                 if size > cfg.movie_rename_limit.get_int():
                     if 'sample' not in file:
-                        tmp, ext = os.path.splitext(file)
-                        self.fname = tmp
+                        self.fname, ext = os.path.splitext(os.path.split(file)[1])
                         newname = "%s%s" % (self.filename_set, ext)
-                        newname = newname.replace('%fn', tmp)
+                        newname = newname.replace('%fn', self.fname)
                         newpath = os.path.join(current_path, newname)
                         if not os.path.exists(newpath):
                             try:
@@ -898,7 +900,7 @@ class DateSorter(object):
                             except:
                                 logging.error(Ta('Failed to rename: %s to %s'), current_path, newpath)
                                 logging.info("Traceback: ", exc_info = True)
-                            rename_similar(current_path, ext, self.filename_set)
+                            rename_similar(current_path, ext, self.filename_set, ())
                             break
 
 
@@ -1091,10 +1093,11 @@ def strip_folders(path):
     return os.path.normpath('/'.join([strip_all(x) for x in f]))
 
 
-def rename_similar(folder, skip_ext, name):
+def rename_similar(folder, skip_ext, name, skipped_files):
     """ Rename all other files in the 'folder' hierarchy after 'name'
         and move them to the root of 'folder'.
         Files having extension 'skip_ext' will be moved, but not renamed.
+        Don't touch files in list `skipped_files`
     """
     logging.debug('Give files in set "%s" matching names.', name)
     folder = os.path.normpath(folder)
@@ -1103,6 +1106,8 @@ def rename_similar(folder, skip_ext, name):
     for root, dirs, files in os.walk(folder):
         for f in files:
             path = os.path.join(root, f)
+            if path in skipped_files:
+                continue
             org, ext = os.path.splitext(f)
             if ext.lower() == skip_ext:
                 # Move file, but do not rename
